# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-

from lmi.storage.StorageConfiguration import StorageConfiguration
from lmi.storage.SettingManager import SettingManager, Setting
import lmi.providers.cmpi_logging as cmpi_logging
from lmi.providers.TimerManager import TimerManager

import logging
import unittest
import os
import shutil
import time

class TestSetting(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        # fake ProviderEnvironment for TimerManager
        class Env(object):
            def AttachThread(self):
                pass
            def PrepareAttachThread(self):
                return self
            def get_cimom_handle(self):
                return self
        cmpi_logging.logger = logging.getLogger('lmi.storage')
        cls.timer_mgr = TimerManager.get_instance(Env())

    def setUp(self):
        self.directory = os.path.dirname(__file__)
        if not self.directory:
            self.directory = "."


        StorageConfiguration.CONFIG_FILE = "/not/existing"
        self.config = StorageConfiguration()

    def test_missing(self):
        """
            Test loading persistent and preconfigured setting when appropriate
            directories are missing.
        """
        self.config.CONFIG_PATH = self.directory + "/configs/missing/etc/"
        self.config.PERSISTENT_PATH = self.directory + "/configs/missing/var/"

        mgr = SettingManager(self.config, self.timer_mgr)
        mgr.load()

        self.assertDictEqual(mgr.classes, {})

    def test_empty(self):
        """
            Test loading persistent and preconfigured setting when appropriate
            directories are empty.
        """
        self.config.CONFIG_PATH = self.directory + "/configs/empty/etc/"
        self.config.PERSISTENT_PATH = self.directory + "/configs/empty/var/"

        mgr = SettingManager(self.config, self.timer_mgr)
        mgr.load()

        self.assertDictEqual(mgr.classes, {})

    def test_full(self):
        """
            Test loading persistent and preconfigured setting when appropriate
            directories are empty.
        """
        self.config.CONFIG_PATH = self.directory + "/configs/full/etc/"
        self.config.PERSISTENT_PATH = self.directory + "/configs/full/var/"

        mgr = SettingManager(self.config, self.timer_mgr)
        mgr.load()

        # check LMI_StorageSetting class loaded OK
        self.assertTrue(mgr.classes.has_key("LMI_StorageSetting"))
        # check it has all instances
        settings = mgr.get_settings("LMI_StorageSetting")
        self.assertIn("LMI:StorageSetting:preconfigured1", settings.keys())
        self.assertIn("LMI:StorageSetting:preconfigured2", settings.keys())
        self.assertIn("LMI:StorageSetting:persistent1", settings.keys())
        self.assertIn("LMI:StorageSetting:persistent2", settings.keys())
        self.assertEqual(len(settings.keys()), 4)

        # check one preconfigured setting
        s1 = settings['LMI:StorageSetting:preconfigured1']
        self.assertEqual(s1.the_id, "LMI:StorageSetting:preconfigured1")
        self.assertEqual(s1.type, Setting.TYPE_PRECONFIGURED)
        self.assertEqual(s1['first'], "1")
        self.assertEqual(s1['second'], "two")
        self.assertEqual(s1['third'], "3.0")

        # check one persistent setting
        s2 = settings['LMI:StorageSetting:persistent2']
        self.assertEqual(s2.the_id, "LMI:StorageSetting:persistent2")
        self.assertEqual(s2.type, Setting.TYPE_PERSISTENT)
        self.assertEqual(s2['first'], "1000")
        self.assertEqual(s2['second'], "two thousand")
        self.assertEqual(s2['third'], "3000.0")

    def test_save_load(self):
        """ Test saving a persistent settings and loading them back."""
        # load the 'full' settings
        self.config.CONFIG_PATH = self.directory + "/configs/full/etc/"
        self.config.PERSISTENT_PATH = self.directory + "/configs/full/var/"

        mgr = SettingManager(self.config, self.timer_mgr)
        mgr.load()

        # dirty hack to save it to different directory...
        self.config.PERSISTENT_PATH = self.directory + "/configs/save_load/var/"

        # add one transient setting
        s = Setting(mgr, 'LMI_StorageSetting',
                Setting.TYPE_TRANSIENT, "LMI:StorageSetting:transient1")
        s['first'] = "111"
        s['second'] = "two two two"
        s['third'] = "333.0"
        mgr.set_setting("LMI_StorageSetting", s)

        # add one preconfigured setting (this should not happen in reality,
        # but let's test it).
        s = Setting(mgr, 'LMI_StorageSetting',
                Setting.TYPE_PRECONFIGURED, "LMI:StorageSetting:preconfigured3")
        s['first'] = "1111"
        s['second'] = "two two two two"
        s['third'] = "3333.0"
        mgr.set_setting("LMI_StorageSetting", s)

        # add one persistent setting
        s = Setting(mgr, 'LMI_StorageSetting',
                Setting.TYPE_PERSISTENT, "LMI:StorageSetting:persistent3")
        s['first'] = "11"
        s['second'] = "two two"
        s['third'] = "33.0"
        mgr.set_setting("LMI_StorageSetting", s)

        # the persistent setting should be saved
        # try to reload the cofig - it should remove the preconfigured one
        mgr.load()

        # check it has all instances and that the preconfigured is gone
        settings = mgr.get_settings("LMI_StorageSetting")
        self.assertIn("LMI:StorageSetting:preconfigured1", settings.keys())
        self.assertIn("LMI:StorageSetting:preconfigured2", settings.keys())
        self.assertIn("LMI:StorageSetting:persistent1", settings.keys())
        self.assertIn("LMI:StorageSetting:persistent2", settings.keys())
        self.assertIn("LMI:StorageSetting:persistent3", settings.keys())
        self.assertIn("LMI:StorageSetting:transient1", settings.keys())
        self.assertEqual(len(settings.keys()), 6)

        # check the transient is ok
        s1 = settings['LMI:StorageSetting:transient1']
        self.assertEqual(s1.the_id, "LMI:StorageSetting:transient1")
        self.assertEqual(s1.type, Setting.TYPE_TRANSIENT)
        self.assertEqual(s1['first'], "111")
        self.assertEqual(s1['second'], "two two two")
        self.assertEqual(s1['third'], "333.0")

        # check the persistent is there
        s2 = settings['LMI:StorageSetting:persistent3']
        self.assertEqual(s2.the_id, "LMI:StorageSetting:persistent3")
        self.assertEqual(s2.type, Setting.TYPE_PERSISTENT)
        self.assertEqual(s2['first'], "11")
        self.assertEqual(s2['second'], "two two")
        self.assertEqual(s2['third'], "33.0")

        # remove one persistent, it should be saved imediatelly
        mgr.delete_setting('LMI_StorageSetting', s2)
        # check it is really removed
        mgr = SettingManager(self.config, self.timer_mgr)
        mgr.load()
        settings = mgr.get_settings("LMI_StorageSetting")
        self.assertNotIn("LMI:StorageSetting:persistent3", settings.keys())

        # change one persistent, it should be saved imediatelly
        s3 = settings['LMI:StorageSetting:persistent2']
        s3['first'] = "-1"
        s3['second'] = "minus one"
        s3['third'] = "-3.0"
        mgr.set_setting('LMI_StorageSetting', s3)
        # check it is really removed
        mgr = SettingManager(self.config, self.timer_mgr)
        mgr.load()
        settings = mgr.get_settings("LMI_StorageSetting")
        s3 = settings['LMI:StorageSetting:persistent2']
        self.assertEqual(s3.the_id, "LMI:StorageSetting:persistent2")
        self.assertEqual(s3.type, Setting.TYPE_PERSISTENT)
        self.assertEqual(s3['first'], "-1")
        self.assertEqual(s3['second'], "minus one")
        self.assertEqual(s3['third'], "-3.0")

    def test_expire(self):
        """
        Test that transient setting expires.
        """
        # hack shorter timeout
        old_timeout = Setting.TRAINSIENT_SETTING_LIFETIME
        Setting.TRAINSIENT_SETTING_LIFETIME = 3

        self.config.CONFIG_PATH = self.directory + "/configs/none/etc/"
        self.config.PERSISTENT_PATH = self.directory + "/configs/none/var/"

        try:
            mgr = SettingManager(self.config, self.timer_mgr)

            # add one transient setting
            s = Setting(mgr, 'LMI_StorageSetting',
                    Setting.TYPE_TRANSIENT, "LMI:StorageSetting:transient1")
            s['first'] = "111"
            s['second'] = "two two two"
            s['third'] = "333.0"
            mgr.set_setting("LMI_StorageSetting", s)

            # add one preconfigured setting (this should not happen in reality,
            # but let's test it).
            s = Setting(mgr, 'LMI_StorageSetting',
                    Setting.TYPE_PRECONFIGURED, "LMI:StorageSetting:preconfigured3")
            s['first'] = "1111"
            s['second'] = "two two two two"
            s['third'] = "3333.0"
            mgr.set_setting("LMI_StorageSetting", s)

            # add one persistent setting
            s = Setting(mgr, 'LMI_StorageSetting',
                    Setting.TYPE_PERSISTENT, "LMI:StorageSetting:persistent3")
            s['first'] = "11"
            s['second'] = "two two"
            s['third'] = "33.0"
            mgr.set_setting("LMI_StorageSetting", s)

            time.sleep(4)

            settings = mgr.get_settings("LMI_StorageSetting")
            self.assertIn('LMI:StorageSetting:persistent3', settings.keys())
            self.assertIn('LMI:StorageSetting:preconfigured3', settings.keys())
            self.assertNotIn(
                    'LMI:StorageSetting:transient1', settings.keys())

            # change the persistent to transient -> must be deleted
            setting = settings['LMI:StorageSetting:persistent3']
            setting.type = Setting.TYPE_TRANSIENT
            mgr.set_setting('LMI_StorageSetting', setting)

            # add transient
            s = Setting(mgr, 'LMI_StorageSetting',
                    Setting.TYPE_TRANSIENT, "LMI:StorageSetting:transient4")
            s['first'] = "111"
            s['second'] = "two two two"
            s['third'] = "333.0"
            mgr.set_setting("LMI_StorageSetting", s)
            # and change it to persistent -> must not be deleted
            s.type = Setting.TYPE_PERSISTENT
            mgr.set_setting("LMI_StorageSetting", s)

            time.sleep(4)

            settings = mgr.get_settings("LMI_StorageSetting")
            self.assertNotIn('LMI:StorageSetting:persistent3', settings.keys())
            self.assertIn('LMI:StorageSetting:preconfigured3', settings.keys())
            self.assertIn('LMI:StorageSetting:transient4', settings.keys())

            setting = settings['LMI:StorageSetting:transient4']
            mgr.delete_setting('LMI_StorageSetting', setting)

        finally:
            Setting.TRAINSIENT_SETTING_LIFETIME = old_timeout

    def tearDown(self):
        # remove any files in configs/save_load/var/
        path = self.directory + "/configs/save_load/var/"
        shutil.rmtree(path, ignore_errors=True)



if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>

from test_base import StorageTestBase
import unittest
import pywbem

class TestDisk(StorageTestBase):
    """
        Test basic features of hard disks.
         - it can be enumerated
         - it can be get
         - it cannot be removed
         - it cannot be modified
    """

    DISK_CLASS = "LMI_StorageExtent"

    def _check_name(self, diskname):
        """ Check that CIMInstanceName represents a disk. """
        self.assertEqual(diskname['SystemName'], self.SYSTEM_NAME)
        self.assertEqual(diskname['SystemCreationClassName'], self.SYSTEM_CLASS_NAME)
        self.assertEqual(diskname['CreationClassName'], self.DISK_CLASS)

    def test_enumerate_instances(self):
        """ Test EnumerateInstances on disks and their properties. """
        disks = self.wbemconnection.EnumerateInstances(self.DISK_CLASS)
        disknames = [disk.path for disk in disks]
        self.assertGreater(len(disks), 1)

        # check that all disks were returned
        self.assertCIMNameIn(self.disk_name, disknames)

        # now, check common properties
        for disk in disks:
            if disk['DeviceID'] != self.disk_name['DeviceID']:
                continue

            self.assertEqual(disk['ExtentStripeLength'], 1)
            self.assertEqual(disk['DataRedundancy'], 1)
            self.assertEqual(disk['PackageRedundancy'], 0)
            self.assertEqual(disk['NoSinglePointOfFailure'], False)
            self.assertEqual(disk['IsComposite'], False)
            self.assertEqual(disk['Primordial'], True)

            # TODO: check exact values
            self.assertGreater(disk['NumberOfBlocks'], 0)
            self.assertGreater(disk['ConsumableBlocks'], 0)
            self.assertGreater(disk['BlockSize'], 0)


    def test_enumerate_names(self):
        """ Test EnumerateInstanceNames on disks. """
        disknames = self.wbemconnection.EnumerateInstanceNames(self.DISK_CLASS)
        self.assertGreater(len(disknames), 1)

        self.assertCIMNameIn(self.disk_name, disknames)

        # check that every disk can be get
        for diskname in disknames:
            disk = self.wbemconnection.GetInstance(diskname)
            self.assertIsNotNone(disk)
            if diskname['DeviceID'] == self.disk_name['DeviceID']:
                self._check_name(disk.path)

    def _create_disk_name(self, device_id):
        """ Return CIMInstanceName for given DeviceID."""
        name = pywbem.CIMInstanceName(
                classname=self.DISK_CLASS,
                keybindings={
                        'DeviceID': device_id,
                        'SystemCreationClassName': self.SYSTEM_CLASS_NAME,
                        'SystemName': self.SYSTEM_NAME,
                        'CreationClassName': self.DISK_CLASS})
        return name

    def test_get(self):
        name = self._create_disk_name(self.disk_name['DeviceID'])
        disk = self.wbemconnection.GetInstance(name)
        self.assertIsNotNone(disk)

    def test_remove(self):
        name = self._create_disk_name(self.disk_name['DeviceID'])
        self.assertRaises(pywbem.CIMError,
                self.wbemconnection.DeleteInstance, name)

    def test_modify(self):
        name = self._create_disk_name(self.disk_name['DeviceID'])
        disk = self.wbemconnection.GetInstance(name)
        self.assertRaises(pywbem.CIMError,
                self.wbemconnection.ModifyInstance, disk)

if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>

from test_base import StorageTestBase
import unittest
import pywbem

class TestSetPartitionStyle(StorageTestBase):
    """
        Test LMI_DiskPartitionConfigurationService.SetPartitionStyle
        with different parameters.
    """

    MBR_CLASS = "LMI_DiskPartition"
    GPT_CLASS = "LMI_GenericDiskPartition"

    STYLE_EMBR = 4100
    STYLE_MBR = 2
    STYLE_GPT = 3

    def setUp(self):
        """ Find disk partition service. """
        super(TestSetPartitionStyle, self).setUp()
        self.service = self.wbemconnection.EnumerateInstanceNames(
                "LMI_DiskPartitionConfigurationService")[0]

    def _check_name(self, partname, classname):
        """ Check that CIMInstanceName represents a partition. """
        self.assertEqual(partname['SystemName'], self.SYSTEM_NAME)
        self.assertEqual(partname['SystemCreationClassName'], self.SYSTEM_CLASS_NAME)
        self.assertEqual(partname['CreationClassName'], classname)

    def _check_capabilities_name(self, capabilities_name, partition_style):
        """
            Check that given partition capabilities name represent givent
            partition_style.
        """
        capabilities = self.wbemconnection.GetInstance(capabilities_name)
        self.assertEqual(capabilities[0]['PartitionStyle'], partition_style)


    def _check_partition_table_type(self, diskname, partition_style):
        """
            Check that disk (represented by CIMInstanceName diskname)
            has partition table partition_style.
        """
        capabilities = self.wbemconnection.Associators(
                diskname,
                AssocClass="LMI_InstalledPartitionTable")
        # there must be only one associated Capabilities instance
        self.assertEqual(len(capabilities), 1)
        self.assertEqual(capabilities[0]['PartitionStyle'], partition_style)

    def _get_capabilities_name(self, style):
        """
            Return CIMInstanceName with partition capabilities representing
            given partition style.
        """
        if style == self.STYLE_EMBR:
            style_name = "EMBR"
        elif style == self.STYLE_MBR:
            style_name = "MBR"
        elif style == self.STYLE_GPT:
            style_name = "GPT"

        return pywbem.CIMInstanceName(
                classname="LMI_DiskPartitionConfigurationCapabilities",
                keybindings={
                        'InstanceID': "LMI:LMI_DiskPartitionConfigurationCapabilities:" + style_name
                })

    def test_no_params(self):
        """ Try SetPartitionStyle with no parameters"""
        # Extent = None -> error
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "SetPartitionStyle",
                self.service)

    def test_no_partition_style(self):
        """ Test SetPartitionStyle with no PartitionStyle parameter."""
        # Extent = sda -> success, with default capabilities (GPT)
        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "SetPartitionStyle",
                self.service,
                Extent=self.disk_name)
        self.assertEqual(retval, 0)
        self._check_partition_table_type(self.disk_name, self.STYLE_GPT)
        self.assertNocaseDictEqual(outparams, {})

    def test_gpt(self):
        """ Test SetPartitionStyle with GPT capabilities."""
        part_style = self._get_capabilities_name(self.STYLE_GPT)
        # Extent = sda, partStyle = MBR -> success
        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "SetPartitionStyle",
                self.service,
                Extent=self.disk_name,
                PartitionStyle=part_style)
        self.assertEqual(retval, 0)
        self._check_partition_table_type(self.disk_name, self.STYLE_GPT)
        self.assertNocaseDictEqual(outparams, {})

    def test_mbr(self):
        """ Test SetPartitionStyle with MBR capabilities."""
        part_style = self._get_capabilities_name(self.STYLE_MBR)
        # Extent = sda, partStyle = MBR -> success
        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "SetPartitionStyle",
                self.service,
                Extent=self.disk_name,
                PartitionStyle=part_style)
        self.assertEqual(retval, 0)
        self._check_partition_table_type(self.disk_name, self.STYLE_MBR)
        self.assertNocaseDictEqual(outparams, {})

    def test_embr(self):
        """ Test SetPartitionStyle with EMBR capabilities."""
        part_style = self._get_capabilities_name(self.STYLE_EMBR)
        # Extent = sda, partStyle = MBR -> success
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "SetPartitionStyle",
                self.service,
                Extent=self.disk_name,
                PartitionStyle=part_style)

    def test_on_partition(self):
        """ Test SetPartitionStyle on a partition."""
        # create a partition table on disk
        part_style = self._get_capabilities_name(self.STYLE_MBR)
        # Extent = sda, partStyle = MBR -> success
        (retval, outparams) = self.wbemconnection.InvokeMethod(
                "SetPartitionStyle",
                self.service,
                Extent=self.disk_name,
                PartitionStyle=part_style)
        self.assertEqual(retval, 0)
        self._check_partition_table_type(self.disk_name, self.STYLE_MBR)
        self.assertNocaseDictEqual(outparams, {})

        # create partition on the disk
        (retval, outparams) = self.invoke_async_method(
                "LMI_CreateOrModifyPartition",
                self.service,
                int, 'partition',
                extent=self.disk_name)
        self.assertEqual(retval, 0)
        partition = outparams['partition']

        # try to create partition table on it
        self.assertRaises(pywbem.CIMError, self.wbemconnection.InvokeMethod,
                "SetPartitionStyle",
                self.service,
                Extent=partition,
                PartitionStyle=part_style)

        # remove the partition
        (ret, outparams) = self.invoke_async_method(
                "LMI_DeletePartition",
                self.service,
                int, None,
                Partition=partition)
        self.assertEquals(ret, 0)


    # TODO: add SetPartitionStyle on RAID, LVM etc.

if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_LUKSBasedOn class.

LMI_LUKSBasedOn
-----------------

.. autoclass:: LMI_LUKSBasedOn
    :members:

"""

from lmi.storage.BasedOnProvider import BasedOnProvider
import pywbem
import lmi.providers.cmpi_logging as cmpi_logging
import blivet

class LMI_LUKSBasedOn(BasedOnProvider):
    """
        Implementation of BasedOn class.
    """
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_LUKSBasedOn, self).__init__(*args, **kwargs)

    @cmpi_logging.trace_method
    def enumerate_devices(self):
        """
            Enumerate all devices, which are in this association as
            Dependent ones, i.e. all devices, which do not have any
            specialized BasedOn class
        """
        for dev in self.storage.devices:
            if isinstance(dev, blivet.devices.LUKSDevice):
                yield dev



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for StorageConfiguration class.

StorageConfiguration
--------------------

.. autoclass:: StorageConfiguration
    :members:

"""

import ConfigParser
import socket
from lmi.base.BaseConfiguration import BaseConfiguration
import lmi.providers.cmpi_logging as cmpi_logging

class StorageConfiguration(BaseConfiguration):
    """
        Configuration class specific to storage providers.
        OpenLMI configuration file should reside in
        /etc/openlmi/storage/storage.conf.
    """

    PERSISTENT_PATH = '/var/lib/openlmi-storage/'
    SETTINGS_DIR = 'settings/'

    @classmethod
    def provider_prefix(cls):
        return "storage"

    @classmethod
    def default_options(cls):
        """ :rtype: (``dict``) Dictionary of default values. """
        defaults = BaseConfiguration.default_options().copy()
        defaults['DebugBlivet'] = 'false'
        return defaults

    @property
    def blivet_tracing(self):
        """ Return True if blivet tracing is enabled."""
        return self.config.getboolean('Log', 'DebugBlivet')




# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
    This module is the main entry from lmi.storage.CIMOM.

    All initialization must be done here.

    This module instantiates all providers and registers them in CIMOM.
"""

from lmi.storage.StorageConfiguration import StorageConfiguration
from lmi.storage.ProviderManager import ProviderManager
from lmi.storage.SettingManager import SettingManager

from lmi.storage.LMI_StorageExtent import LMI_StorageExtent
from lmi.storage.LMI_MDRAIDStorageExtent import LMI_MDRAIDStorageExtent
from lmi.storage.LMI_DiskPartition import LMI_DiskPartition
from lmi.storage.LMI_GenericDiskPartition import LMI_GenericDiskPartition
from lmi.storage.LMI_LVStorageExtent import LMI_LVStorageExtent
from lmi.storage.LMI_VGStoragePool import LMI_VGStoragePool
from lmi.storage.LMI_PartitionBasedOn import LMI_PartitionBasedOn
from lmi.storage.LMI_MDRAIDBasedOn import LMI_MDRAIDBasedOn
from lmi.storage.LMI_LVBasedOn import LMI_LVBasedOn
from lmi.storage.LMI_LVAllocatedFromStoragePool \
        import LMI_LVAllocatedFromStoragePool
from lmi.storage.LMI_VGAssociatedComponentExtent \
        import LMI_VGAssociatedComponentExtent
from lmi.storage.LMI_DiskPartitionConfigurationSetting \
        import LMI_DiskPartitionConfigurationSetting
from lmi.storage.SettingProvider \
        import ElementSettingDataProvider, SettingHelperProvider
from lmi.storage.LMI_DiskPartitioningConfigurationService \
        import LMI_DiskPartitionConfigurationService
from lmi.storage.LMI_HostedStorageService \
        import LMI_HostedStorageService
from lmi.storage.LMI_DiskPartitionConfigurationCapabilities \
        import LMI_DiskPartitionConfigurationCapabilities
from lmi.storage.CapabilitiesProvider import ElementCapabilitiesProvider
from lmi.storage.LMI_InstalledPartitionTable \
        import LMI_InstalledPartitionTable
from lmi.storage.LMI_LVStorageCapabilities \
        import LMI_LVStorageCapabilities, LMI_LVElementCapabilities
from lmi.storage.LMI_StorageConfigurationService \
        import LMI_StorageConfigurationService
from lmi.storage.LMI_VGStorageCapabilities import LMI_VGStorageCapabilities
from lmi.storage.LMI_MDRAIDStorageCapabilities \
        import LMI_MDRAIDStorageCapabilities
from lmi.storage.LMI_SystemStorageDevice import LMI_SystemStorageDevice
from lmi.storage.LMI_MDRAIDFormatProvider import LMI_MDRAIDFormatProvider
from lmi.storage.LMI_PVFormatProvider import LMI_PVFormatProvider
from lmi.storage.LMI_DataFormatProvider import LMI_DataFormatProvider
from lmi.storage.FormatProvider import LMI_ResidesOnExtent
from lmi.storage.LMI_LocalFileSystem import LMI_LocalFileSystem
from lmi.storage.LMI_FileSystemConfigurationService \
        import LMI_FileSystemConfigurationService
from lmi.storage.LMI_FileSystemConfigurationCapabilities \
        import LMI_FileSystemConfigurationCapabilities
from lmi.storage.LMI_FileSystemCapabilities \
        import LMI_FileSystemCapabilities
from lmi.providers.JobManager import JobManager
from lmi.providers.IndicationManager import IndicationManager
from lmi.storage.LMI_HostedFileSystem import LMI_HostedFileSystem
from lmi.storage.LMI_MountedFileSystem import LMI_MountedFileSystem
from lmi.storage.LMI_HostedMount import LMI_HostedMount
from lmi.storage.LMI_MountPoint import LMI_MountPoint
from lmi.storage.LMI_AttachedFileSystem import LMI_AttachedFileSystem
from lmi.storage.LMI_MountConfigurationService import LMI_MountConfigurationService
from lmi.storage.LMI_MountedFileSystemCapabilities import LMI_MountedFileSystemCapabilities
from lmi.providers.TimerManager import TimerManager
from lmi.storage.LMI_TransientFileSystem import LMI_TransientFileSystem
from lmi.storage.LMI_BlockStorageStatisticalData import \
    LMI_BlockStorageStatisticalData, LMI_StorageElementStatisticalData, \
    LMI_StorageStatisticsCollection, LMI_MemberOfStorageStatisticsCollection, \
    LMI_HostedStorageStatisticsCollection, \
    LMI_BlockStatisticsManifestCollection, LMI_BlockStatisticsManifest, \
    LMI_MemberOfBlockStatisticsManifestCollection, \
    LMI_AssociatedBlockStatisticsManifestCollection, \
    LMI_BlockStatisticsService, LMI_BlockStatisticsCapabilities
from lmi.storage.LMI_LUKSStorageExtent import LMI_LUKSStorageExtent
from lmi.storage.LMI_LUKSFormat import LMI_LUKSFormat
from lmi.storage.LMI_ExtentEncryptionConfigurationService import LMI_ExtentEncryptionConfigurationService
from lmi.storage.LMI_LUKSBasedOn import LMI_LUKSBasedOn

import lmi.providers.cmpi_logging as cmpi_logging
from lmi.providers.ComputerSystem import get_system_name
import blivet
import logging

timer_manager = None
indication_manager = None
job_manager = None

LOG = cmpi_logging.get_logger(__name__)

def init_anaconda(config):
    """ Initialize Anaconda storage module."""
    LOG().info("Initializing Anaconda")

    # enable discovery of non-device filesystems (procfs, tmpfs, ...)
    blivet.flags.include_nodev = True
    # set up storage class instance
    storage = blivet.Blivet()
    # identify the system's storage devices
    storage.reset()
    return storage

def get_providers(env):
    """
        CIMOM callback. Initialize OpenLMI and return dictionary of all
        providers we implement.
    """
    # allow **magic here
    # pylint: disable-msg=W0142

    config = StorageConfiguration.get_instance()
    cmpi_logging.setup(env, config)
    root_logger = logging.getLogger()
    anaconda_loggers = [
        logging.getLogger('blivet'),
        logging.getLogger('program')]
    for logger in anaconda_loggers:
        logger.setLevel(logging.DEBUG if config.blivet_tracing else logging.WARNING)
        for handler in root_logger.handlers:
            logger.addHandler(handler)
    LOG().info("Provider init.")

    # Initialize ComputerSystem.Name
    get_system_name(env)

    # initialize the timer manager
    global timer_manager
    timer_manager = TimerManager.get_instance(env)

    global indication_manager
    indication_manager = IndicationManager.get_instance(
            env, "Storage", config.namespace)

    manager = ProviderManager()
    setting_manager = SettingManager(config, timer_manager)
    setting_manager.load()
    storage = init_anaconda(config)

    global job_manager
    job_manager = JobManager('Storage', config.namespace,
            indication_manager, timer_manager)

    providers = {}
    # common construction options
    opts = {'storage': storage,
            'config': config,
            'provider_manager': manager,
            'setting_manager': setting_manager,
            'job_manager' : job_manager}

    # StorageDevice providers
    provider = LMI_StorageExtent(**opts)
    manager.add_device_provider(provider)
    providers['LMI_StorageExtent'] = provider

    provider = LMI_MDRAIDStorageExtent(**opts)
    manager.add_device_provider(provider)
    providers['LMI_MDRAIDStorageExtent'] = provider
    setting_provider = SettingHelperProvider(
            setting_helper=provider,
            setting_classname="LMI_MDRAIDStorageSetting",
            **opts)
    manager.add_setting_provider(setting_provider)
    providers['LMI_MDRAIDStorageSetting'] = setting_provider
    assoc_provider = ElementSettingDataProvider(
            setting_provider=setting_provider,
            managed_element_classname="LMI_MDRAIDStorageExtent",
            setting_data_classname="LMI_MDRAIDStorageSetting",
            **opts)
    providers['LMI_MDRAIDElementSettingData'] = assoc_provider

    provider = LMI_DiskPartition(**opts)
    manager.add_device_provider(provider)
    providers['LMI_DiskPartition'] = provider

    provider = LMI_GenericDiskPartition(**opts)
    manager.add_device_provider(provider)
    providers['LMI_GenericDiskPartition'] = provider

    provider = LMI_LVStorageExtent(**opts)
    manager.add_device_provider(provider)
    providers['LMI_LVStorageExtent'] = provider
    setting_provider = SettingHelperProvider(
            setting_helper=provider,
            setting_classname="LMI_LVStorageSetting",
            **opts)
    manager.add_setting_provider(setting_provider)
    providers['LMI_LVStorageSetting'] = setting_provider
    assoc_provider = ElementSettingDataProvider(
            setting_provider=setting_provider,
            managed_element_classname="LMI_LVStorageExtent",
            setting_data_classname="LMI_LVStorageSetting",
            **opts)
    providers['LMI_LVElementSettingData'] = assoc_provider

    provider = LMI_VGStoragePool(**opts)
    manager.add_device_provider(provider)
    providers['LMI_VGStoragePool'] = provider
    setting_provider = SettingHelperProvider(
            setting_helper=provider,
            setting_classname="LMI_VGStorageSetting",
            **opts)
    manager.add_setting_provider(setting_provider)
    providers['LMI_VGStorageSetting'] = setting_provider
    assoc_provider = ElementSettingDataProvider(
            setting_provider=setting_provider,
            managed_element_classname="LMI_VGStoragePool",
            setting_data_classname="LMI_VGStorageSetting",
            **opts)
    providers['LMI_VGElementSettingData'] = assoc_provider
    cap_provider = LMI_LVStorageCapabilities(**opts)
    manager.add_capabilities_provider(cap_provider)
    providers['LMI_LVStorageCapabilities'] = cap_provider
    assoc_provider = LMI_LVElementCapabilities(
            "LMI_LVElementCapabilities",
            cap_provider, provider, **opts)
    providers['LMI_LVElementCapabilities'] = assoc_provider

    # mounting
    provider = LMI_MountedFileSystem(**opts)
    providers['LMI_MountedFileSystem'] = provider

    setting_provider = SettingHelperProvider(
            setting_helper=provider,
            setting_classname="LMI_MountedFileSystemSetting",
            **opts)
    manager.add_setting_provider(setting_provider)
    providers['LMI_MountedFileSystemSetting'] = setting_provider

    assoc_provider = ElementSettingDataProvider(
            setting_provider=setting_provider,
            managed_element_classname="LMI_MountedFileSystem",
            setting_data_classname="LMI_MountedFileSystemSetting",
            **opts)
    providers['LMI_MountedFileSystemElementSettingData'] = assoc_provider

    provider = LMI_HostedMount(**opts)
    providers['LMI_HostedMount'] = provider

    provider = LMI_MountPoint(**opts)
    providers['LMI_MountPoint'] = provider

    provider = LMI_AttachedFileSystem(**opts)
    providers['LMI_AttachedFileSystem'] = provider

    service_provider = LMI_MountConfigurationService(**opts)
    manager.add_service_provider(service_provider)
    providers['LMI_MountConfigurationService'] = service_provider

    cap_provider = LMI_MountedFileSystemCapabilities(**opts)
    manager.add_capabilities_provider(cap_provider)
    providers['LMI_MountedFileSystemCapabilities'] = cap_provider

    assoc_provider = ElementCapabilitiesProvider(
            'LMI_MountElementCapabilities',
            cap_provider, service_provider, **opts)
    providers['LMI_MountElementCapabilities'] = assoc_provider

    # luks
    provider = LMI_LUKSStorageExtent(**opts)
    manager.add_device_provider(provider)
    providers['LMI_LUKSStorageExtent'] = provider

    provider = LMI_LUKSBasedOn(**opts)
    providers['LMI_LUKSBasedOn'] = provider

    provider = LMI_LUKSFormat(**opts)
    manager.add_format_provider(provider)
    providers['LMI_LUKSFormat'] = provider

    service_provider = LMI_ExtentEncryptionConfigurationService(**opts)
    manager.add_service_provider(service_provider)
    providers['LMI_ExtentEncryptionConfigurationService'] = service_provider

    # settings
    setting_provider = LMI_DiskPartitionConfigurationSetting(
            ** opts)
    manager.add_setting_provider(setting_provider)
    providers['LMI_DiskPartitionConfigurationSetting'] = setting_provider
    assoc_provider = ElementSettingDataProvider(
            setting_provider=setting_provider,
            managed_element_classname="CIM_DiskPartition",
            setting_data_classname="LMI_DiskPartitionConfigurationSetting",
            **opts)
    providers['LMI_DiskPartitionElementSettingData'] = assoc_provider


    # services & capabilities
    service_provider = LMI_StorageConfigurationService(**opts)
    manager.add_service_provider(service_provider)
    providers['LMI_StorageConfigurationService'] = service_provider
    cap_provider = LMI_VGStorageCapabilities(**opts)
    manager.add_capabilities_provider(cap_provider)
    providers['LMI_VGStorageCapabilities'] = cap_provider
    assoc_provider = ElementCapabilitiesProvider(
            "LMI_VGElementCapabilities",
            cap_provider, service_provider, **opts)
    providers['LMI_VGElementCapabilities'] = assoc_provider

    cap_provider = LMI_MDRAIDStorageCapabilities(**opts)
    manager.add_capabilities_provider(cap_provider)
    providers['LMI_MDRAIDStorageCapabilities'] = cap_provider
    assoc_provider = ElementCapabilitiesProvider(
            "LMI_MDRAIDElementCapabilities",
            cap_provider, service_provider, **opts)
    providers['LMI_MDRAIDElementCapabilities'] = assoc_provider


    service_provider = LMI_DiskPartitionConfigurationService(
            ** opts)

    manager.add_service_provider(service_provider)
    providers['LMI_DiskPartitionConfigurationService'] = service_provider

    cap_provider = LMI_DiskPartitionConfigurationCapabilities(
            ** opts)
    manager.add_capabilities_provider(cap_provider)
    providers['LMI_DiskPartitionConfigurationCapabilities'] = cap_provider
    assoc_provider = ElementCapabilitiesProvider(
            "LMI_DiskPartitionElementCapabilities",
            cap_provider, service_provider, **opts)
    providers['LMI_DiskPartitionElementCapabilities'] = assoc_provider


    # Associations
    provider = LMI_PartitionBasedOn(**opts)
    providers['LMI_PartitionBasedOn'] = provider

    provider = LMI_MDRAIDBasedOn(**opts)
    providers['LMI_MDRAIDBasedOn'] = provider

    provider = LMI_LVBasedOn(**opts)
    providers['LMI_LVBasedOn'] = provider

    provider = LMI_LVAllocatedFromStoragePool(**opts)
    providers['LMI_LVAllocatedFromStoragePool'] = provider

    provider = LMI_VGAssociatedComponentExtent(**opts)
    providers['LMI_VGAssociatedComponentExtent'] = provider

    provider = LMI_HostedStorageService(**opts)
    providers['LMI_HostedStorageService'] = provider

    provider = LMI_SystemStorageDevice(**opts)
    providers['LMI_SystemStorageDevice'] = provider

    provider = LMI_InstalledPartitionTable(**opts)
    providers['LMI_InstalledPartitionTable'] = provider

    fmt = LMI_DataFormatProvider(**opts)
    manager.add_format_provider(fmt)
    providers['LMI_DataFormat'] = fmt

    fmt = LMI_MDRAIDFormatProvider(**opts)
    manager.add_format_provider(fmt)
    providers['LMI_MDRAIDFormat'] = fmt

    fmt = LMI_PVFormatProvider(**opts)
    manager.add_format_provider(fmt)
    providers['LMI_PVFormat'] = fmt

    fmt = LMI_LocalFileSystem(**opts)
    manager.add_format_provider(fmt)
    providers['LMI_LocalFileSystem'] = fmt
    setting_provider = SettingHelperProvider(
            setting_helper=fmt,
            setting_classname="LMI_FileSystemSetting",
            **opts)
    manager.add_setting_provider(setting_provider)
    providers['LMI_FileSystemSetting'] = setting_provider
    assoc_provider = ElementSettingDataProvider(
            setting_provider=setting_provider,
            managed_element_classname="LMI_LocalFileSystem",
            setting_data_classname="LMI_FileSystemSetting",
            **opts)
    providers['LMI_FileSystemElementSettingData'] = assoc_provider

    fmt = LMI_TransientFileSystem(**opts)
    manager.add_format_provider(fmt)
    providers['LMI_TransientFileSystem'] = fmt

    service_provider = LMI_FileSystemConfigurationService(**opts)
    manager.add_service_provider(service_provider)
    providers['LMI_FileSystemConfigurationService'] = service_provider
    cap_provider = LMI_FileSystemConfigurationCapabilities(
            ** opts)
    manager.add_capabilities_provider(cap_provider)
    providers['LMI_FileSystemConfigurationCapabilities'] = cap_provider
    assoc_provider = ElementCapabilitiesProvider(
            "LMI_FileSystemConfigurationElementCapabilities",
            cap_provider, service_provider, **opts)
    providers['LMI_FileSystemConfigurationElementCapabilities'] = assoc_provider

    cap_provider = LMI_FileSystemCapabilities(**opts)
    manager.add_capabilities_provider(cap_provider)
    providers['LMI_FileSystemCapabilities'] = cap_provider
    assoc_provider = ElementCapabilitiesProvider(
            "LMI_FileSystemElementCapabilities",
            cap_provider, service_provider, **opts)
    providers['LMI_FileSystemElementCapabilities'] = assoc_provider

    provider = LMI_ResidesOnExtent(**opts)
    providers['LMI_ResidesOnExtent'] = provider

    job_providers = job_manager.get_providers()
    providers.update(job_providers)

    provider = LMI_HostedFileSystem(**opts)
    providers['LMI_HostedFileSystem'] = provider

    # Block Statistics providers
    block_stat_provider = LMI_BlockStorageStatisticalData(**opts)
    providers['LMI_BlockStorageStatisticalData'] = block_stat_provider

    assoc_provider = LMI_StorageElementStatisticalData(
            block_stat_provider=block_stat_provider,
            **opts)
    providers['LMI_StorageElementStatisticalData'] = assoc_provider

    provider = LMI_StorageStatisticsCollection(**opts)
    providers['LMI_StorageStatisticsCollection'] = provider

    provider = LMI_MemberOfStorageStatisticsCollection(
            block_stat_provider=block_stat_provider,
            **opts)
    providers['LMI_MemberOfStorageStatisticsCollection'] = provider

    provider = LMI_HostedStorageStatisticsCollection(**opts)
    providers['LMI_HostedStorageStatisticsCollection'] = provider

    provider = LMI_BlockStatisticsManifestCollection(**opts)
    providers['LMI_BlockStatisticsManifestCollection'] = provider

    manifest_provider = LMI_BlockStatisticsManifest(**opts)
    providers['LMI_BlockStatisticsManifest'] = manifest_provider

    provider = LMI_MemberOfBlockStatisticsManifestCollection(
            manifest_provider=manifest_provider, **opts)
    providers['LMI_MemberOfBlockStatisticsManifestCollection'] = provider

    provider = LMI_AssociatedBlockStatisticsManifestCollection(**opts)
    providers['LMI_AssociatedBlockStatisticsManifestCollection'] = provider

    service_provider = LMI_BlockStatisticsService(
            block_stat_provider,
            **opts)
    providers['LMI_BlockStatisticsService'] = service_provider
    manager.add_service_provider(service_provider)

    cap_provider = LMI_BlockStatisticsCapabilities(**opts)
    providers['LMI_BlockStatisticsCapabilities'] = cap_provider

    assoc_provider = ElementCapabilitiesProvider(
            "LMI_BlockStorageStatisticsElementCapabilities",
            cap_provider, service_provider, **opts)
    providers['LMI_BlockStorageStatisticsElementCapabilities'] = assoc_provider

    LOG().trace_info("Registered providers: %s", str(providers))
    return providers

def authorize_filter(env, fltr, ns, classes, owner):
    """ CIMOM callback."""
    IndicationManager.get_instance().authorize_filter(
            env, fltr, ns, classes, owner)

def activate_filter (env, fltr, ns, classes, first_activation):
    """ CIMOM callback."""
    IndicationManager.get_instance().activate_filter(
            env, fltr, ns, classes, first_activation)

def deactivate_filter(env, fltr, ns, classes, last_activation):
    """ CIMOM callback."""
    IndicationManager.get_instance().deactivate_filter(
            env, fltr, ns, classes, last_activation)

def enable_indications(env):
    """ CIMOM callback."""
    IndicationManager.get_instance().enable_indications(env)

def disable_indications(env):
    """ CIMOM callback."""
    IndicationManager.get_instance().disable_indications(env)

def can_unload(_env):
    """ CIMOM callback."""
    LOG().trace_info("can_unload called")
    return job_manager.can_shutdown()

def shutdown(_env):
    """ CIMOM callback."""
    LOG().info("Provider shutdown.")
    # Job manager must be the first, it uses TimerManager and IndicationManager
    job_manager.shutdown()

    timer_manager.shutdown()
    indication_manager.shutdown()



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
""" Module for LMI_BlockStorageStatisticalData."""

import pywbem
from lmi.storage.BaseProvider import BaseProvider
import lmi.providers.cmpi_logging as cmpi_logging
import lmi.storage.util.storage as storage
from lmi.providers import parse_instance_id
from lmi.storage.ServiceProvider import ServiceProvider
from lmi.storage.CapabilitiesProvider import CapabilitiesProvider
from lmi.providers.ComputerSystem import get_system_name
import datetime

LOG = cmpi_logging.get_logger(__name__)

class LMI_BlockStorageStatisticalData(BaseProvider):
    """
    Provider for LMI_BlockStorageStatisticalData.
    """

    # Expected nr. of columns in /sys/block/xxx/stat
    STAT_ITEM_COUNT = 11
    # Indexes to self._current_stats
    # Number of read I/Os processed
    STAT_READ_COUNT = 0
    # Number of sectors read, in 512 bytes sectors!
    STAT_READ_SECTORS = 2
    # Total wait time for read requests, in milliseconds
    # Beware, this number is multiplied by nr. of waiting requests ->
    # not usable for LMI_BlockStorageStatisticalData.
    STAT_READ_TICKS = 3
    # Number of write I/Os processed
    STAT_WRITE_COUNT = 4
    # Number of sectors written, in 512 bytes sectors!
    STAT_WRITE_SECTORS = 6
    # Total wait time for write requests, in milliseconds
    # Beware, this number is multiplied by nr. of waiting requests ->
    # not usable for LMI_BlockStorageStatisticalData.
    STAT_WRITE_TICKS = 7
    # Total wait time for all requests.
    # This is real time, not multiplied by nr. of requests.
    STAT_ALL_TICKS = 9

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_BlockStorageStatisticalData, self).__init__(*args, **kwargs)

    @cmpi_logging.trace_method
    def has_statistics(self, device, broker):
        """
        Determine, if given device should have LMI_BlockStorageStatisticalData
        instance associated.
        :param device: (``StorageDevice``) Device to examine.
        :param broker: (``CIMOMHandle``) CIMOM broker to use, we need to call
            is_subclass().
        """
        devname = self.provider_manager.get_name_for_device(device)
        if not devname:
            return False
        if not broker.is_subclass(
                self.config.namespace,
                "CIM_StorageExtent",
                devname.classname):
            # We provide statistics only for StorageExtents and not Pools
            return False
        return True

    @cmpi_logging.trace_method
    def load_stats(self, device):
        """
        Load statistics from the device.

        :param device; (``StorageDevice``) Device to measure.
        :returns: dictionary property name (string) -> property value.
        """
        statname = "/sys" + device.sysfsPath + "/stat"
        try:
            with open(statname, "rt") as f:
                line = f.readline()
        except IOError:
            # Translate IOError to CIMError to give user more specific message.
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Cannot read statistics for device %s"
                    % device.path)

        stats = line.split()
        if len(stats) < self.STAT_ITEM_COUNT:
            LOG().trace_warn("Cannot parse statistics from %s, got '%s'",
                    statname, line)
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Cannot parse statistics for device %s"
                    % device.path)

        # Convert the values to integers
        stats = map(int, stats)

        model = pywbem.cim_obj.NocaseDict()
        model['ElementName'] = device.path
        model['StatisticTime'] = pywbem.CIMDateTime(datetime.datetime.utcnow())
        model['ElementType'] = self.Values.ElementType.Extent

        # Don't forget to convert sectors to KBytes and milliseconds to
        # hundreds of ms.
        model['IOTimeCounter'] = pywbem.Uint64(
                stats[self.STAT_ALL_TICKS] / 100)
        model['KBytesRead'] = pywbem.Uint64(
                stats[self.STAT_READ_SECTORS] / 2)
        model['KBytesWritten'] = pywbem.Uint64(
                stats[self.STAT_WRITE_SECTORS] / 2)
        model['ReadIOs'] = pywbem.Uint64(stats[self.STAT_READ_COUNT])
        model['TotalIOs'] = pywbem.Uint64(
                stats[self.STAT_READ_COUNT]
                + stats[self.STAT_WRITE_COUNT])
        model['WriteIOs'] = pywbem.Uint64(stats[self.STAT_WRITE_COUNT])
        model['KBytesTransferred'] = pywbem.Uint64(
                (stats[self.STAT_READ_SECTORS]
                        + stats[self.STAT_WRITE_SECTORS]) / 2)
        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            EnumerateInstances provider method.
        """
        model.path.update({'InstanceID': None})
        broker = env.get_cimom_handle()
        for device in self.storage.devices:
            if not self.has_statistics(device, broker):
                continue
            name = storage.get_persistent_name(device)
            model['InstanceID'] = "LMI:LMI_BlockStorageStatisticalData:" + name

            if keys_only:
                yield model
            else:
                yield self.get_instance(env, model, device)

    @cmpi_logging.trace_method
    def get_device_for_name(self, name):
        """
        Find StorageDevice for given InstanceName. Return None if there is no such
        device.
        :param name: (``CIMInstanceName`` or ``CIMInstance``) InstanceName to
            examine.
        :returns: ``StorageDevice`` Appropriate device or None if the device
            is not found.
        """
        _id = parse_instance_id(
                name['InstanceID'], "LMI_BlockStorageStatisticalData")
        if not _id:
            return None
        device = storage.get_device_for_persistent_name(self.storage, _id)
        return device

    @cmpi_logging.trace_method
    def get_name_for_device(self, device):
        """
        Create CIMInstanceName of LMI_BlockStorageStatisticalData for given
        device.
        :param device: (``StorageDevice`` Device to get name from.
        :returns:  (``CIMInstanceName``) InstanceName, which refers to stats
            of the device.
        """
        name = storage.get_persistent_name(device)
        return pywbem.CIMInstanceName(
                classname="LMI_BlockStorageStatisticalData",
                namespace=self.config.namespace,
                keybindings={
                        'InstanceID':
                            "LMI:LMI_BlockStorageStatisticalData:" + name
                })

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0221
    def get_instance(self, env, model, device=None):
        """
            GetInstance method provider.
        """
        if not device:
            device = self.get_device_for_name(model)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find device for this InstanceID.")

        stats = self.load_stats(device)
        model.update(stats)
        return model

    class Values(object):
        class ElementType(object):
            Computer_System = pywbem.Uint16(2)
            Front_end_Computer_System = pywbem.Uint16(3)
            Peer_Computer_System = pywbem.Uint16(4)
            Back_end_Computer_System = pywbem.Uint16(5)
            Front_end_Port = pywbem.Uint16(6)
            Back_end_Port = pywbem.Uint16(7)
            Volume = pywbem.Uint16(8)
            Extent = pywbem.Uint16(9)
            Disk_Drive = pywbem.Uint16(10)
            Arbitrary_LUs = pywbem.Uint16(11)
            Remote_Replica_Group = pywbem.Uint16(12)
            # DMTF_Reserved = ..
            # Vendor_Specific = 0x8000..

class LMI_StorageElementStatisticalData(BaseProvider):
    """
    Provider for LMI_BlockStorageStatisticalData.
    """
    @cmpi_logging.trace_method
    def __init__(self, block_stat_provider, *args, **kwargs):
        """
        :param block_stat_provider: (``LMI_BlockStorageStatisticalData``)
            Provider instance to use.
        """
        self.block_stat_provider = block_stat_provider
        super(LMI_StorageElementStatisticalData, self).__init__(*args, **kwargs)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            EnumerateInstances provider method.
        """
        model.path.update({'ManagedElement': None, 'Stats': None})
        broker = env.get_cimom_handle()
        for device in self.storage.devices:
            if not self.block_stat_provider.has_statistics(device, broker):
                continue
            model['Stats'] = self.block_stat_provider.get_name_for_device(
                    device)
            model['ManagedElement'] = self.provider_manager.get_name_for_device(
                    device)
            yield model

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_instance(self, env, model):
        """
            GetInstance method provider.
        """
        # Just check that the keys are correct
        device_name = model['ManagedElement']
        device = self.provider_manager.get_device_for_name(device_name)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find device for ManagedElement.")

        broker = env.get_cimom_handle()
        if not self.block_stat_provider.has_statistics(device, broker):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "The ManagedElement has no statistics.")

        device2 = self.block_stat_provider.get_device_for_name(model['Stats'])
        if device != device2:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "The ManagedElement is not related to Stats.")
        return model

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations."""
        return self.simple_references(env, object_name, model,
                result_class_name, role, result_role, keys_only,
                "CIM_StorageExtent",
                "LMI_BlockStorageStatisticalData")

class LMI_StorageStatisticsCollection(BaseProvider):
    """
    Provider of LMI_StorageStatisticsCollection. There is only one instance
    of LMI_StorageStatisticsCollection on the system.
    """
    INSTANCE_ID = "LMI:LMI_StorageStatisticsCollection:instance"

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_StorageStatisticsCollection, self).__init__(*args, **kwargs)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            EnumerateInstances provider method.
        """
        model.path.update({'InstanceID': None})
        model['InstanceID'] = self.INSTANCE_ID
        if keys_only:
            yield model
        else:
            yield self.get_instance(env, model)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_instance(self, env, model):
        """
            GetInstance method provider.
        """
        if model['InstanceID'] != self.INSTANCE_ID:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Unknown InstanceID.")
        # 0 = no fixed sampling interval
        model['SampleInterval'] = pywbem.CIMDateTime(datetime.timedelta(0))
        return model

class LMI_MemberOfStorageStatisticsCollection(BaseProvider):
    @cmpi_logging.trace_method
    def __init__(self, block_stat_provider, *args, **kwargs):
        """
        :param block_stat_provider: (``LMI_BlockStorageStatisticalData``)
            Provider instance to use.
        """
        self.block_stat_provider = block_stat_provider
        super(LMI_MemberOfStorageStatisticsCollection, self).__init__(
                *args, **kwargs)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            EnumerateInstances provider method.
        """
        model.path.update({'Collection': None, 'Member': None})
        broker = env.get_cimom_handle()
        for device in self.storage.devices:
            if not self.block_stat_provider.has_statistics(device, broker):
                continue
            model['Collection'] = pywbem.CIMInstanceName(
                    classname="LMI_StorageStatisticsCollection",
                    namespace=self.config.namespace,
                    keybindings={
                        'InstanceID' :
                            LMI_StorageStatisticsCollection.INSTANCE_ID
                    })
            model['Member'] = self.block_stat_provider.get_name_for_device(
                    device)
            yield model

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_instance(self, env, model):
        """
            GetInstance method provider.
        """
        # Just check instance keys
        if model['Collection'] ['InstanceID'] != \
                LMI_StorageStatisticsCollection.INSTANCE_ID:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Unknown Collection InstanceID.")
        device = self.block_stat_provider.get_device_for_name(model['Member'])
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Unknown Member.")
        broker = env.get_cimom_handle()
        if not self.block_stat_provider.has_statistics(device, broker):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Member is not part of the Collection.")
        return model

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations."""
        return self.simple_references(env, object_name, model,
                result_class_name, role, result_role, keys_only,
                "LMI_BlockStorageStatisticalData",
                "LMI_StorageStatisticsCollection")

class LMI_HostedStorageStatisticsCollection(BaseProvider):
    """
    Provider of LMI_HostedStorageStatisticsCollection. There is only one instance
    of LMI_HostedStorageStatisticsCollection on the system.
    """

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_HostedStorageStatisticsCollection, self).__init__(
                *args, **kwargs)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            EnumerateInstances provider method.
        """
        model.path.update({'Antecedent': None, 'Dependent': None})
        model['Antecedent'] = pywbem.CIMInstanceName(
                    classname=self.config.system_class_name,
                    namespace=self.config.namespace,
                    keybindings={
                            'CreationClassName' : self.config.system_class_name,
                            'Name' : get_system_name(),
                    })

        model['Dependent'] = pywbem.CIMInstanceName(
                    classname="LMI_StorageStatisticsCollection",
                    namespace=self.config.namespace,
                    keybindings={
                        'InstanceID' :
                            LMI_StorageStatisticsCollection.INSTANCE_ID
                    })
        yield model

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_instance(self, env, model):
        """
            GetInstance method provider.
        """
        # just check keys
        system = model['Antecedent']
        if (system['CreationClassName'] != self.config.system_class_name
                or system['Name'] != get_system_name()):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Wrong Antecedent keys.")
        if model['Dependent'] ['InstanceID'] != \
                LMI_StorageStatisticsCollection.INSTANCE_ID:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Unknown Dependent InstanceID.")
        return model

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations."""
        return self.simple_references(env, object_name, model,
                result_class_name, role, result_role, keys_only,
                "CIM_System",
                "LMI_StorageStatisticsCollection")

class LMI_BlockStatisticsManifestCollection(BaseProvider):
    """
    Provider of LMI_BlockStatisticsManifestCollection. There is only one instance
    of LMI_BlockStatisticsManifestCollection on the system, we don't allow
    user-specified manifests.
    """
    INSTANCE_ID = "LMI:LMI_BlockStatisticsManifestCollection:instance"

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_BlockStatisticsManifestCollection, self).__init__(
                *args, **kwargs)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            EnumerateInstances provider method.
        """
        model.path.update({'InstanceID': None})
        model['InstanceID'] = self.INSTANCE_ID
        if keys_only:
            yield model
        else:
            yield self.get_instance(env, model)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_instance(self, env, model):
        """
            GetInstance method provider.
        """
        if model['InstanceID'] != self.INSTANCE_ID:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Unknown InstanceID.")
        model['IsDefault'] = True
        return model

class LMI_BlockStatisticsManifest(BaseProvider):
    """
    Provider of LMI_BlockStatisticsManifest. We don't allow
    user-specified manifests.
    """

    # Sequence of columns in CSV format returned by
    # LMI_BlockStatisticsService.GetStatisticsCollection().
    CSV_SEQUENCE = [
            'InstanceID',
            'ElementType',
            'StatisticTime',
            'IOTimeCounter',
            'KBytesRead',
            'KBytesTransferred',
            'KBytesWritten',
            'ReadIOs',
            'TotalIOs',
            'WriteIOs',
        ]

    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_BlockStatisticsManifest, self).__init__(
                *args, **kwargs)
        # The manifests are static, we keep them in dictionary
        # InstanceID -> { property name -> property value }
        extent = LMI_BlockStorageStatisticalData.Values.ElementType.Extent
        self.manifests = {
                'LMI:LMI_BlockStatisticsManifest:Extent' : {
                    'ElementType': extent,
                    'IncludeIdleTimeCounter': False,
                    'IncludeIOTimeCounter': True,
                    'IncludeKBytesRead': True,
                    'IncludeKBytesTransferred': True,
                    'IncludeKBytesWritten': True,
                    'IncludeMaintOp': False,
                    'IncludeMaintTimeCounter': False,
                    'IncludeReadHitIOs': False,
                    'IncludeReadHitIOTimeCounter': False,
                    'IncludeReadIOs': True,
                    'IncludeReadIOTimeCounter': False,
                    'IncludeStartStatisticTime': False,
                    'IncludeStatisticTime': True,
                    'IncludeTotalIOs': True,
                    'IncludeWriteHitIOs': False,
                    'IncludeWriteHitIOTimeCounter': False,
                    'IncludeWriteIOs': True,
                    'IncludeWriteIOTimeCounter': False,
                    'CSVSequence': self.CSV_SEQUENCE
                }
        }

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            EnumerateInstances provider method.
        """
        model.path.update({'InstanceID': None})
        for key in self.manifests.iterkeys():
            model['InstanceID'] = key
            if keys_only:
                yield model
            else:
                yield self.get_instance(env, model)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_instance(self, env, model):
        """
            GetInstance method provider.
        """
        manifest = self.manifests.get(model['InstanceID'], None)
        if not manifest:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Unknown InstanceID.")
        model.update(manifest)
        return model

class LMI_MemberOfBlockStatisticsManifestCollection(BaseProvider):
    """
    Provider of LMI_MemberOfBlockStatisticsManifestCollection.
    """
    @cmpi_logging.trace_method
    def __init__(self, manifest_provider, *args, **kwargs):
        """
        :param manifest_provider: (``LMI_BlockStatisticsManifest``)
            Provider instance to use.
        """
        self.manifest_provider = manifest_provider
        super(LMI_MemberOfBlockStatisticsManifestCollection, self).__init__(
                *args, **kwargs)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            EnumerateInstances provider method.
        """
        model.path.update({'Collection': None, 'Member': None})

        manifest_model = pywbem.CIMInstance(
                classname='LMI_BlockStatisticsManifest',
                path=pywbem.CIMInstanceName(
                        classname='LMI_BlockStatisticsManifest',
                        namespace=self.config.namespace))

        for manifest in self.manifest_provider.enum_instances(
                env, manifest_model, True):
            model['Collection'] = pywbem.CIMInstanceName(
                    classname="LMI_BlockStatisticsManifestCollection",
                    namespace=self.config.namespace,
                    keybindings={
                        'InstanceID' :
                            LMI_BlockStatisticsManifestCollection.INSTANCE_ID
                    })
            model['Member'] = manifest.path
            yield model

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_instance(self, env, model):
        """
            GetInstance method provider.
        """
        # Just check instance keys
        if model['Collection'] ['InstanceID'] != \
                LMI_BlockStatisticsManifestCollection.INSTANCE_ID:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Unknown Collection InstanceID.")
        # get_instance will throw Not Found error if the Member is wrong:
        # (use Member copy to avoid modification of 'model')
        _manifest_model = self.manifest_provider.get_instance(
                env, model['Member'].copy())
        return model

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations."""
        return self.simple_references(env, object_name, model,
                result_class_name, role, result_role, keys_only,
                "LMI_BlockStatisticsManifestCollection",
                "LMI_BlockStatisticsManifest")

class LMI_AssociatedBlockStatisticsManifestCollection(BaseProvider):
    """
    Provider of LMI_AssociatedBlockStatisticsManifestCollection.
    There should be only one LMI_AssociatedBlockStatisticsManifestCollection
    instance, since we have only one ManifestCollection.
    """
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_AssociatedBlockStatisticsManifestCollection, self).__init__(
                *args, **kwargs)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            EnumerateInstances provider method.
        """
        model.path.update({'Statistics': None, 'ManifestCollection': None})

        model['ManifestCollection'] = pywbem.CIMInstanceName(
                classname="LMI_BlockStatisticsManifestCollection",
                namespace=self.config.namespace,
                keybindings={
                    'InstanceID' :
                        LMI_BlockStatisticsManifestCollection.INSTANCE_ID
                })
        model['Statistics'] = pywbem.CIMInstanceName(
                    classname="LMI_StorageStatisticsCollection",
                    namespace=self.config.namespace,
                    keybindings={
                        'InstanceID' :
                            LMI_StorageStatisticsCollection.INSTANCE_ID
                    })
        yield model

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def get_instance(self, env, model):
        """
            GetInstance method provider.
        """
        # Just check instance keys
        if model['Statistics'] ['InstanceID'] != \
                LMI_StorageStatisticsCollection.INSTANCE_ID:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Unknown Dependent InstanceID.")
        if model['ManifestCollection'] ['InstanceID'] != \
                LMI_BlockStatisticsManifestCollection.INSTANCE_ID:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Unknown Dependent InstanceID.")
        return model

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations."""
        return self.simple_references(env, object_name, model,
                result_class_name, role, result_role, keys_only,
                "LMI_BlockStatisticsManifestCollection",
                "LMI_StorageStatisticsCollection")

class LMI_BlockStatisticsService(ServiceProvider):
    """
        LMI_BlockStatisticsService provider implementation.
    """
    # Maximum size of Statistics[] item in GetStatisticsCollection() method.
    # SMI-S says it should be less than 64 kB
    MAXCHUNK = 65535

    @cmpi_logging.trace_method
    def __init__(self, block_stat_provider, *args, **kwargs):
        """
        :param block_stat_provider: (``LMI_BlockStorageStatisticalData``)
            Provider instance to use.
        """
        self.block_stat_provider = block_stat_provider
        super(LMI_BlockStatisticsService, self).__init__(
                "LMI_BlockStatisticsService", *args, **kwargs)

    @cmpi_logging.trace_method
    def _get_statistics_line(self, device):
        """
        Return line for Statistics[] parameter of
        LMI_BlockStatisticsService.GetStatisticsCollection()
        for given device.

        The line is semicolon-separated list of values of properties as listed
        in associated LMI_BlockStatisticsManifest.

        :param device: (``StorageDevice``) Device to examine.
        :returns: ``string`` with device statistics.
        """
        stats = self.block_stat_provider.load_stats(device)
        # Add missing InstanceID property
        name = storage.get_persistent_name(device)
        stats['InstanceID'] = "LMI:LMI_BlockStorageStatisticalData:" + name

        columns = []
        for name in LMI_BlockStatisticsManifest.CSV_SEQUENCE:
            columns.append(str(stats[name]))
        return ";".join(columns)

    @cmpi_logging.trace_method
    def cim_method_getstatisticscollection(self, env, object_name,
                                           param_manifestcollection=None,
                                           param_statisticsformat=None,
                                           param_elementtypes=None,
                                           param_statistics=None):
        """Implements LMI_BlockStatisticsService.GetStatisticsCollection()

        Retrieves statistics in a well-defined bulk format. The collection
        of statistics returned is determined by the list of element types
        passed in to the method and the manifests for those types
        contained in the supplied BlockStatisticsManifestCollection. If
        both the Elements and BlockStatisticsManifestCollection parameters
        are supplied, then the types of elements returned is an
        intersection of the element types listed in the Elements parameter
        and the types for which BlockStatisticsManifest instances exist in
        the supplied BlockStatisticsManifestCollection. The statistics are
        returned through a well-defined array of strings, whose format is
        specified by the StatisticsFormat parameter, that can be parsed to
        retrieve the desired statistics as well as limited information
        about the elements that those metrics describe.

        param_manifestcollection --  The input parameter ManifestCollection (type REF (pywbem.CIMInstanceName(classname='CIM_BlockStatisticsManifestCollection', ...)) 
            The BlockStatisticsManifestCollection that contains the
            manifests that list the metrics to be returned for each
            element type. If not supplied (i.e. parameter is null), then
            all available statistics will be returned unfiltered. Only
            elements that match the element type properties (if
            meaningful) of the BlockStatisticsManifest instances contained
            within the BlockStatisticsManifestCollection will have data
            returned by this method. If the supplied
            BlockStatisticsManifestCollection does not contain any
            BlockStatisticsManifest instances, then no statistics will be
            returned by this method.
        param_statisticsformat --  The input parameter StatisticsFormat (type pywbem.Uint16 self.Values.GetStatisticsCollection.StatisticsFormat) 
            Specifies the format of the Statistics output parameter.  - CSV
            = Comma Separated Values.
        param_elementtypes --  The input parameter ElementTypes (type [pywbem.Uint16,] self.Values.GetStatisticsCollection.ElementTypes) 
            Element types for which statistics should be returned. If not
            supplied (i.e. parameter is null) this parameter is not
            considered when filtering the instances of StatisticalData
            that will populate the Statistics output parameter. If the
            array is not null, but is empty, then no statistics will be
            returned by this method. A client SHOULD NOT specify this
            parameter if it is not meaningful (i.e. the service only
            provides statistics for a single type of element).

        Output parameters:
        Job -- (type REF (pywbem.CIMInstanceName(classname='CIM_ConcreteJob', ...)) 
            Reference to the job (may be null if job completed).

        Statistics -- (type [unicode,]) 
            The statistics for all the elements as determined by the
            Elements, ManifestCollection parameters, and StatisticsFormat
            parameters.
        """
        # check the parameters
        self.check_instance(object_name)
        if param_manifestcollection:
            if (param_manifestcollection['InstanceID'] !=
                    LMI_BlockStatisticsManifestCollection.INSTANCE_ID):
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                        "Unknown ManifestCollection.")
        csv = self.Values.GetStatisticsCollection.StatisticsFormat.CSV
        if (param_statisticsformat is not None
                and param_statisticsformat != csv):
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "Parameter StatisticsFormat must be CSV.")

        extent = self.Values.GetStatisticsCollection.ElementTypes.Extent
        if (param_elementtypes is not None
                and param_elementtypes != [extent]):
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "Parameter ElementTypes must be [Extent].")

        # Finally, get the statistics.
        broker = env.get_cimom_handle()
        # The output parameter:
        statistics = []
        # Current chunk, candidate to be inserted to statistics[]:
        chunk = ""

        for device in self.storage.devices:
            if not self.block_stat_provider.has_statistics(device, broker):
                continue

            try:
                line = self._get_statistics_line(device)
            except pywbem.CIMError:
                # Ignore any errors, the device may have disappeared
                # -> no line in resulting statistics
                LOG().trace_warn("Skipping device %s in statistics.",
                        device.path)
                continue

            if len(line) + len(chunk) + 1 > self.MAXCHUNK:
                statistics.append(chunk)
                chunk = ""
            chunk += line + "\n"

        statistics.append(chunk)
        out_params = []
        out_params += [pywbem.CIMParameter('statistics', type='string',
                           value=statistics)]
        rval = self.Values.GetStatisticsCollection.Job_Completed_with_No_Error
        return (rval, out_params)

    class Values(ServiceProvider.Values):
        class GetStatisticsCollection(object):
            Job_Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            # Method_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Element_Not_Supported = pywbem.Uint32(4097)
            Statistics_Format_Not_Supported = pywbem.Uint32(4098)
            # Method_Reserved = 4099..32767
            # Vendor_Specific = 32768..65535
            class StatisticsFormat(object):
                Unknown = pywbem.Uint16(0)
                Other = pywbem.Uint16(1)
                CSV = pywbem.Uint16(2)
                # DMTF_Reserved = ..
                # Vendor_Specific = 0x8000..
            class ElementTypes(object):
                Computer_System = pywbem.Uint16(2)
                Front_end_Computer_System = pywbem.Uint16(3)
                Peer_Computer_System = pywbem.Uint16(4)
                Back_end_Computer_System = pywbem.Uint16(5)
                Front_end_Port = pywbem.Uint16(6)
                Back_end_Port = pywbem.Uint16(7)
                Volume = pywbem.Uint16(8)
                Extent = pywbem.Uint16(9)
                Disk_Drive = pywbem.Uint16(10)
                Arbitrary_LUs = pywbem.Uint16(11)
                Remote_Replica_Group = pywbem.Uint16(12)
                # DMTF_Reserved = ..
                # Vendor_Specific = 0x8000..


class LMI_BlockStatisticsCapabilities(CapabilitiesProvider):
    """
    LMI_DiskPartitionConfigurationCapabilities provider implementation.
    """
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_BlockStatisticsCapabilities, self).__init__(
                "LMI_BlockStatisticsCapabilities", *args, **kwargs)
        element_types = LMI_BlockStorageStatisticalData.Values.ElementType
        methods = self.Values.SynchronousMethodsSupported
        features = self.Values.SupportedFeatures
        self.instance = {
                'InstanceID': 'LMI:LMI_BlockStatisticsCapabilities:instance',
                'ElementTypesSupported': [pywbem.Uint16(element_types.Extent)],
                'SynchronousMethodsSupported': [
                        methods.GetStatisticsCollection],
                'AsynchronousMethodsSupported': pywbem.CIMProperty(
                            name='AsynchronousMethodsSupported',
                            value=[],
                            type='uint16',
                            array_size=0,
                            is_array=True),
                'ClockTickInterval': pywbem.Uint64(100000),  # 100 milliseconds
                'SupportedFeatures': [features.none],
                'ElementName': "BlockStatisticsCapabilities",
                '_default': True
        }
    @cmpi_logging.trace_method
    def enumerate_capabilities(self):
        """
            Return an iterable with all capabilities instances, i.e.
            dictionaries property_name -> value.
            If the capabilities are the default ones, it must have
            '_default' as a property name.
        """
        return [self.instance]

    @cmpi_logging.trace_method
    def create_setting_for_capabilities(self, capabilities):
        """
            Create LMI_*Setting for given capabilities.
            Return CIMInstanceName of the setting or raise CIMError on error.
        """
        # There is no LMI_BlockStorageStatisticsSetting
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                "This method is not supported.")

    class Values:
        class SynchronousMethodsSupported(object):
            Execute_Query = pywbem.Uint16(2)
            Query_Collection = pywbem.Uint16(3)
            GetStatisticsCollection = pywbem.Uint16(4)
            Manifest_Creation = pywbem.Uint16(5)
            Manifest_Modification = pywbem.Uint16(6)
            Manifest_Removal = pywbem.Uint16(7)
        class SupportedFeatures(object):
            none = pywbem.Uint16(2)
            Client_Defined_Sequence = pywbem.Uint16(3)
            # DMTF_Reserved = ..
            # Vendor_Specific = 0x8000..



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_LVStorageCapabilities class.

LMI_LVStorageCapabilities
-------------------------

.. autoclass:: LMI_LVStorageCapabilities
    :members:

LMI_LVElementCapabilities
-------------------------

.. autoclass:: LMI_LVElementCapabilities
    :members:

"""

from lmi.storage.CapabilitiesProvider import CapabilitiesProvider
import lmi.providers.cmpi_logging as cmpi_logging
from lmi.storage.SettingManager import StorageSetting
import pywbem
import blivet.devices
from lmi.storage.BaseProvider import BaseProvider
from lmi.storage.util import storage
import lmi.providers

LOG = cmpi_logging.get_logger(__name__)

class LMI_LVStorageCapabilities(CapabilitiesProvider):
    """ Provider of LMI_LVStorageCapabilities class."""
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_LVStorageCapabilities, self).__init__(
                classname='LMI_LVStorageCapabilities', *args, **kwargs)
        self.pool_provider = None

    @cmpi_logging.trace_method
    def get_pool_name_for_capabilities(self, instance_id):
        """
            Return CIMInstanceName of storage pool associated with capabilities
            with given id.
        """
        path = lmi.providers.parse_instance_id(instance_id, self.classname)
        if not path:
            return None
        device = storage.get_device_for_persistent_name(self.storage, path)
        if not device:
            return None
        if not isinstance(device,
                blivet.devices.LVMVolumeGroupDevice):
            LOG().trace_warn("InstanceID %s is not LVMVolumeGroupDevice",
                    instance_id)
            return None

        if not self.pool_provider:
            self.pool_provider = self.provider_manager.get_provider_for_device(
                    device)
        return self.pool_provider.get_name_for_device(device)


    @cmpi_logging.trace_method
    def _get_capabilities_for_device(self, device):
        """ Return capabilities fir given device. """
        if not self.pool_provider:
            self.pool_provider = self.provider_manager.get_provider_for_device(
                    device)

        redundancy = self.pool_provider.get_redundancy(device)
        caps = {}
        _id = storage.get_persistent_name(device)
        caps['InstanceID'] = self.create_capabilities_id(_id)
        caps['ElementName'] = device.path
        caps['DataRedundancyDefault'] = \
                pywbem.Uint16(redundancy.data_redundancy)
        caps['DataRedundancyMax'] = pywbem.Uint16(redundancy.data_redundancy)
        caps['DataRedundancyMin'] = pywbem.Uint16(redundancy.data_redundancy)
        caps['NoSinglePointOfFailure'] = redundancy.no_single_point_of_failure
        caps['NoSinglePointOfFailureDefault'] = \
                redundancy.no_single_point_of_failure
        caps['ExtentStripeLengthDefault'] = \
                pywbem.Uint16(redundancy.stripe_length)
        caps['PackageRedundancyDefault'] = \
                pywbem.Uint16(redundancy.package_redundancy)
        caps['PackageRedundancyMax'] = pywbem.Uint16(
                redundancy.package_redundancy)
        caps['PackageRedundancyMin'] = pywbem.Uint16(
                redundancy.package_redundancy)
        if redundancy.parity_layout:
            caps['ParityLayoutDefault'] = pywbem.Uint16(
                    redundancy.parity_layout + 1)
        else:
            caps['ParityLayoutDefault'] = pywbem.CIMProperty(
                    "ParityLayoutDefault",
                    None,
                    type="uint16")
        return caps


    @cmpi_logging.trace_method
    def enumerate_capabilities(self):
        """
            Return an iterable with all capabilities instances, i.e.
            dictionaries property_name -> value.
            If the capabilities are the default ones, it must have
            '_default' as a property name.

            Subclasses must override this method.
        """
        for vg in self.storage.vgs:
            yield self._get_capabilities_for_device(vg)

    @cmpi_logging.trace_method
    def create_setting_for_capabilities(self, capabilities):
        """
            Create LMI_*Setting for given capabilities.
            Return CIMInstanceName of the setting or raise CIMError on error.
       """
        setting_id = self.setting_manager.allocate_id(
                'LMI_LVStorageSetting')
        if not setting_id:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Failed to allocate setting InstanceID")

        setting = self.setting_manager.create_setting(
                'LMI_LVStorageSetting',
                StorageSetting.TYPE_TRANSIENT,
                setting_id,
                class_to_create=StorageSetting)
        setting['DataRedundancyGoal'] = capabilities['DataRedundancyDefault']
        setting['DataRedundancyMax'] = capabilities['DataRedundancyDefault']
        setting['DataRedundancyMin'] = capabilities['DataRedundancyDefault']
        setting['ExtentStripeLength'] = \
                capabilities['ExtentStripeLengthDefault']
        setting['ExtentStripeLengthMax'] = \
                capabilities['ExtentStripeLengthDefault']
        setting['ExtentStripeLengthMin'] = \
                capabilities['ExtentStripeLengthDefault']
        setting['NoSinglePointOfFailure'] = \
                capabilities['NoSinglePointOfFailure']
        setting['PackageRedundancyGoal'] = \
                capabilities['PackageRedundancyDefault']
        setting['PackageRedundancyMax'] = \
                capabilities['PackageRedundancyDefault']
        setting['PackageRedundancyMin'] = \
                capabilities['PackageRedundancyDefault']
        setting['ElementName'] = 'CreatedFrom' + capabilities['InstanceID']

        layout = capabilities['ParityLayoutDefault']
        if isinstance(layout, pywbem.CIMProperty):
            layout_value = layout.value
        else:
            layout_value = capabilities['ParityLayoutDefault']
        if layout_value:
            # StorageSetting.ParityLayout ValueMap is shifted by one
            setting['ParityLayout'] = layout_value - 1
        else:
            setting['ParityLayout'] = None

        self.setting_manager.set_setting('LMI_LVStorageSetting', setting)
        return pywbem.CIMInstanceName(
                classname='LMI_LVStorageSetting',
                namespace=self.config.namespace,
                keybindings={'InstanceID': setting_id})

    @cmpi_logging.trace_method
    def get_capabilities_for_id(self, instance_id):
        """
            Return dictionary property_name -> value.
            If the capabilities are the default ones, it must have
            '_default' as a property name.
            Return None if there is no such Capabilities instance.

            Subclasses can override this method.
        """
        path = lmi.providers.parse_instance_id(instance_id, self.classname)
        if not path:
            return None
        device = storage.get_device_for_persistent_name(self.storage, path)
        if not device:
            return None
        if not isinstance(device,
                blivet.devices.LVMVolumeGroupDevice):
            LOG().trace_warn("InstanceID %s is not LVMVolumeGroupDevice",
                    instance_id)
            return None

        return self._get_capabilities_for_device(device)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0221
    def cim_method_createsetting(self, env, object_name,
            param_settingtype=None):
        """
            Implements LMI_LVStorageCapabilities.CreateSetting()

            Create LMI_LVStorageSetting applicable to this VG
            All properties its will have default values.

        """
        # just check param_settingtype
        if param_settingtype:
            setting_type = self.Values.CreateSetting.SettingType
            if (param_settingtype != setting_type.Default
                    and param_settingtype != setting_type.Goal):
                raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Wrong value of SettingType parameter.")

        (retval, outvars) = super(LMI_LVStorageCapabilities, self)\
                .cim_method_createsetting(env, object_name)

        # rename output param from 'setting' to 'newsetting'
        if outvars and outvars[0].name == "setting":
            outvars = [pywbem.CIMParameter('newsetting', type='reference',
                    value=outvars[0].value)]
        return (retval, outvars)


    @cmpi_logging.trace_method
    def cim_method_createlvstoragesetting(self, env, object_name):
        """
           Implements LMI_LVStorageCapabilities.CreateLVStorageSetting()

            This method creates new instance of LMI_LVStorageSetting.
            Applications then do not need to calculate DataRedundancy,
            PackageRedundancy and ExtentStripeLength. Because only basic
            Logical Volumes without any additional stripping or mirroring are
            supported, this method basically clones LMI_VGStorageSetting to
            LMI_LVStorageSetting.
        """
        return super(LMI_LVStorageCapabilities, self).cim_method_createsetting(
                env, object_name)


    class Values(CapabilitiesProvider.Values):
        class CreateSetting(object):
            Success = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unspecified_Error = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            # DMTF_Reserved = ..
            # Vendor_Specific = 32768..65535
            class SettingType(object):
                Default = pywbem.Uint16(2)
                Goal = pywbem.Uint16(3)

class LMI_LVElementCapabilities(BaseProvider):
    """
        Base class for LMI_*ElementCapabilities providers.

        If all capabilities instances are associated only with appropriate
        LMI_*Service, this class does not need to be subclasses.

        Otherwise, subclasses can associate capabilities to other managed
        elements.
    """
    @cmpi_logging.trace_method
    def __init__(self, classname, capabilities_provider, device_provider,
            *args, **kwargs):
        self.classname = classname
        self.capabilities_provider = capabilities_provider
        self.device_provider = device_provider
        super(LMI_LVElementCapabilities, self).__init__(*args, **kwargs)

    @cmpi_logging.trace_method
    def enumerate_capabilities(self):
        """
            Return iterable with (managed_element_name, capabilities_name),
            where managed_element_name and capabilities_name
            are CIMInstanceName.

            By default, all capabilities provided by capabilities_provider
            are associated to service_provider.

            Subclasses can override this method if different behavior is
            requested.
        """
        for capabilities in self.capabilities_provider.enumerate_capabilities():

            provider = self.capabilities_provider
            managed_element_name = provider.get_pool_name_for_capabilities(
                    capabilities['InstanceID'])

            capabilities_name = pywbem.CIMInstanceName(
                    classname=self.capabilities_provider.classname,
                    namespace=self.config.namespace,
                    keybindings={'InstanceID' : capabilities['InstanceID']})
            yield (managed_element_name, capabilities_name)

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """
            Provider implementation of EnumerateInstances intrinsic method.
        """
        model.path.update({'Capabilities': None, 'ManagedElement': None})
        for (managed_element, capabilities) in self.enumerate_capabilities():
            model['Capabilities'] = capabilities
            model['ManagedElement'] = managed_element
            if keys_only:
                yield model
            else:
                yield self.get_instance(env, model, capabilities)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0221
    def get_instance(self, env, model, capabilities_name=None):
        """
            Provider implementation of GetInstance intrinsic method.
        """
        # find the capabilities instance
        instance_id = None
        if not capabilities_name:
            capabilities = self.capabilities_provider.get_capabilities_for_id(
                    model['Capabilities']['InstanceID'])
            if not capabilities:
                raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                        "Capabilities not found.")
            instance_id = capabilities['InstanceID']
        else:
            instance_id = capabilities_name['InstanceID']

        name = self.capabilities_provider.get_pool_name_for_capabilities(
                instance_id)
        if not name:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "Cannot find StoragePool for Capabilities.")

        if name['InstanceID'] != model['ManagedElement']['InstanceID']:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "ManagedElement is not associated to Capabilities.")

        return model

    @cmpi_logging.trace_method
    def references(self, env, object_name, model, result_class_name, role,
                   result_role, keys_only):
        """Instrument Associations. """
        return self.simple_references(env, object_name, model,
                result_class_name, role, result_role, keys_only,
                "CIM_Capabilities",
                "CIM_ManagedElement")


    class Values(object):
        class Characteristics(object):
            Default = pywbem.Uint16(2)
            Current = pywbem.Uint16(3)
            # DMTF_Reserved = ..
            # Vendor_Specific = 32768..65535



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for LMI_DiskPartitionConfigurationService class.

LMI_DiskPartitionConfigurationService
-------------------------------------

.. autoclass:: LMI_DiskPartitionConfigurationService
    :members:

"""

from lmi.storage.ServiceProvider import ServiceProvider
from lmi.storage.LMI_DiskPartitionConfigurationSetting \
        import LMI_DiskPartitionConfigurationSetting
import pywbem
import blivet.formats
import lmi.storage.util.storage as storage
import lmi.storage.util.units as units
import parted
import lmi.providers.cmpi_logging as cmpi_logging
from lmi.providers.JobManager import Job

class LMI_DiskPartitionConfigurationService(ServiceProvider):
    """
        LMI_DiskPartitionConfigurationService provider implementation.
    """
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_DiskPartitionConfigurationService, self).__init__(
                "LMI_DiskPartitionConfigurationService", *args, **kwargs)

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        model = super(LMI_DiskPartitionConfigurationService, self).get_instance(
                env, model)
        schemes = self.Values.PartitioningSchemes
        model['PartitioningSchemes'] = \
                schemes.Volumes_may_be_partitioned_or_treated_as_whole
        return model

    @cmpi_logging.trace_method
    def cim_method_setpartitionstyle(self, env, object_name,
                                     param_extent=None,
                                     param_partitionstyle=None):
        """
            Implements LMI_DiskPartitionConfigurationService.SetPartitionStyle()

            This method installs a partition table on an extent of the
            specified partition style; creating an association between the
            extent and that capabilities instances referenced as method
            parameters. As a side effect, the consumable block size of the
            underlying extent is reduced by the block size of the metadata
            reserved by the partition table and associated metadata. This size
            is in the PartitionTableSize property of the associated
            DiskPartitionConfigurationCapabilities instance.
        """
        # check parameters here, the real work is done in _setpartitionstyle
        self.check_instance(object_name)

        if not param_extent:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Parameter Extent is mandatory.")

        # check the device
        device = self.provider_manager.get_device_for_name(param_extent)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Cannot find the Extent.")
        if isinstance(device, blivet.devices.PartitionDevice):
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Creation of extended partitions is not supported.")
        # Check the device is unused
        storage.assert_unused(self.storage, [device])

        # check the capabilities
        mgr = self.provider_manager
        capabilities_provider = mgr.get_capabilities_provider_for_class(
                'LMI_DiskPartitionConfigurationCapabilities')
        if not capabilities_provider:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Cannot find capabilities provider.")
        if param_partitionstyle:
            capabilities = capabilities_provider.get_capabilities_for_id(
                    param_partitionstyle['InstanceID'])
            if not capabilities:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "Cannot find capabilities for given PartitionStyle.")
        else:
            # find the default capabilities
            capabilities = capabilities_provider.get_default_capabilities()
            if not capabilities:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "Parameter PartitionStyle is mandatory, there is no"\
                        " default PartitionStyle.")

        part_styles = capabilities_provider.Values.PartitionStyle
        if capabilities['PartitionStyle'] == part_styles.EMBR:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Creation of extended partitions is not supported.")

        retval = self._setpartitionstyle(
                device, capabilities, capabilities_provider)
        return (retval, [])

    @cmpi_logging.trace_method
    def _setpartitionstyle(self, device, capabilities, capabilities_provider):
        """
            Really set the partition style, all parameters were successfully
            checked.
        """
        part_styles = capabilities_provider.Values.PartitionStyle
        if capabilities['PartitionStyle'] == part_styles.MBR:
            label = "msdos"
        elif capabilities['PartitionStyle'] == part_styles.GPT:
            label = "gpt"
        else:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Unsupported PartitionStyle:"
                    + str(capabilities['PartitionStyle']) + ".")

        storage.log_storage_call("CREATE DISKLABEL",
                {'label': label, 'device': device.path})

        fmt = blivet.formats.getFormat('disklabel', labelType=label)
        action = blivet.deviceaction.ActionCreateFormat(device, fmt)
        storage.do_storage_action(self.storage, [action])

        return self.Values.SetPartitionStyle.Success


    @cmpi_logging.trace_method
    def _parse_goal(self, param_goal):
        """
            Check Goal parameter of a CIM method.
            It must be CIMInstanceName of LMI_DiskPartitionConfigurationSetting.
            Return Setting appropriate to the CIMInstanceName or None
            if no Goal was specified.
            Raise CIMError, if the goal does not exist.

            param_goal = CIMInstanceName
        """
        if not param_goal:
            return None

        instance_id = param_goal['InstanceID']
        goal = self.provider_manager.get_setting_for_id(
            instance_id, "LMI_DiskPartitionConfigurationSetting")
        if not goal:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "LMI_DiskPartitionConfigurationSetting Goal does not"
                " found.")
        goal.touch()
        return goal

    @cmpi_logging.trace_method
    def _parse_partition(self, param_partition, device=None):
        """
            Check Partition parameter of a CIM method.
            It must be CIMInstanceName of a CIM_Partition.
            If a device was specified in the method, then check that the
            device contains the partition.
            Return PartitionDevice. Return None if no Partition parameter
            was set.
            Raise CIMError on any error like the partition does not exist
            or the partition is not on given device.

            param_partition = CIMInstanceName
            device = StorageDevice
        """
        if not param_partition:
            return None

        partition = self.provider_manager.get_device_for_name(param_partition)
        if not partition:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                "Cannot find the Partition.")
        if not isinstance(partition, blivet.devices.PartitionDevice):
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "Parameter Partition does not refer to partition.")
        if device:
            # the partition must be on the device
            if device not in partition.parents:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "The Partition does not reside on the Extent.")
        return partition

    @cmpi_logging.trace_method
    def _parse_extent(self, param_extent, goal):
        """
            Check Extent parameter of a CIM method.
            It must be CIMInstanceName of a CIM_StorageExtent and it must
            be possible to create partitions on it.
            If a Goal was specified in the method, then check that the
            partition with such goal can be created on the partition.
            Return (StorageDevice, logical), where 'logical' is True, if
            a logical partition is requested. Return None if no Extent parameter
            was set.
            Raise CIMError on any error, like the device does not exist
            or there is no partition table on the device.

            param_extent = CIMInstanceName
            goal = Setting
        """
        if not param_extent:
            return (None, None)

        logical = False
        device = self.provider_manager.get_device_for_name(param_extent)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                     "Cannot find the Extent.")
        if isinstance(device, blivet.devices.PartitionDevice):
            values = LMI_DiskPartitionConfigurationSetting.Values.PartitionType
            if goal and int(goal['PartitionType']) != values.Logical:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Only Goal with PartitionType == Logical can be"
                    " created on this device.")
            device = device.parents[0]
            logical = True
        return (device, logical)


    # Too many arguments, but this is generated function!
    # pylint: disable-msg=R0913
    def cim_method_createormodifypartition(self, env, object_name,
                                           param_partition=None,
                                           param_devicefilename=None,
                                           param_extent=None,
                                           param_startingaddress=None,
                                           param_endingaddress=None):
        """
            Implements LMI_DiskPartitionConfigurationService.CreateOrModifyPartition()

            This method creates a new partition if the Partition parameter is
            null or modifies the partition specified. If the starting and
            ending address parameters are null, the resulting partition will
            occupy the entire underlying extent. If the starting address is
            non-null and the ending address is null, the resulting partition
            will extend to the end of the underlying extent. \n\nIn
            contradiction to SMI-S, no LogicalDisk will be created on the
            partition.\nIf logical partition is being created, it's start/end
            sector must include space for partition metadata and any alignment
            sectors. ConsumableSpace of the logical partition will be reduced
            by these metadata and alignment sectors.\nThe underlying extent
            MUST be associated to a capabilities class describing the
            installed partition style (partition table); this association is
            established using SetPartitionStyle().
        """
        # check parameters
        self.check_instance(object_name)

        if not param_partition and not param_extent:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Either Partition or extent parameter must be present.")
        if param_devicefilename:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                    "Parameter DeviceFileName is not supported.")

        (device, logical) = self._parse_extent(param_extent, None)
        if logical:
            # Recalculate start/end addresses from 'relative to extended
            # partition start' to 'relative to disk start'
            address_shift = partitioning.get_logical_partition_start(device)
            if param_startingaddress is not None:
                param_startingaddress = param_startingaddress + address_shift
            if param_endingaddress is not None:
                param_endingaddress = param_endingaddress + address_shift

        if device:
            (minstart, maxend) = partitioning.get_available_sectors(device)

        # check partition
        partition = self._parse_partition(param_partition, device)
        if partition:
            if param_startingaddress is None:
                start = 0
            if param_endingaddress is None:
                end = 0
            (retval, partition) = self._modify_partition(
                    partition, None, start, end)
        else:
            if param_startingaddress is None:
                start = minstart
            if param_endingaddress is None:
                end = maxend
            (retval, partition) = self._create_partition(
                    device, None, start, end)

        partition_name = self.provider_manager.get_name_for_device(partition)
        out_params = [pywbem.CIMParameter('partition', type='reference',
                           value=partition_name)]
        return (retval, out_params)

    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def _modify_partition(self, partition, goal, start, end):
        """
            Modify partition to given goal, start and end.
            Start and End can be 0, which means no change.
            Return (retval, partition).
        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                "Partition modification is not supported.")


    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def _lmi_modify_partition(self, partition, goal, size):
        """
            Modify partition to given goal and size.
            Size can be Null, which means no change.
            Return (retval, partition, size).
        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                "Partition modification is not supported.")

    @cmpi_logging.trace_method
    def _get_max_partition_size(self, device, partition_type):
        """
            Return maximum partition size on given device, in bytes.
            Partition_type must be parted constant or None
        """
        if partition_type is None:
            # Find the largest logical and normal and return the largest
            max_primary = self._get_max_partition_size(
                device, parted.PARTITION_NORMAL)
            max_logical = 0
            if device.format.extendedPartition is not None:
                max_logical = self._get_max_partition_size(
                    device, parted.PARTITION_LOGICAL)
            elif (device.format.labelType == 'msdos' and
                    len(device.format.partitions) > 3):
                # There is no extended partition and the new one
                # will be logical
                # -> reserve 2 MB for extended partition metadata
                max_primary = max_primary - 2 * units.MEGABYTE
            return max(max_primary, max_logical)
        else:
            parted_disk = device.format.partedDisk
            geom = blivet.partitioning.getBestFreeSpaceRegion(
                    parted_disk,
                    partition_type,
                    1,
                    grow=True)
            if geom is None:
                return 0
            return geom.getLength()



    @cmpi_logging.trace_method
    # pylint: disable-msg=W0613
    def _create_partition(self, device, goal, start, end):
        """
            Create partition on given device with  given goal, start and end.
            Return (retval, partition).
        """
        # TODO: wait for anaconda to implement start/end sectors
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED,
                "CreateOrModifyPartition is not supported, use"\
                " LMI_CreateOrModifyPartition instead.")

    @cmpi_logging.trace_method
    def _calculate_partition_type(self, device, goal):
        """
            Calculate the right partition type for given goal and return
            pypaterd partition type.
        """
        part_type = None
        part_types = LMI_DiskPartitionConfigurationSetting.Values.PartitionType
        if int(goal['PartitionType']) == part_types.Extended:
            if device.format.labelType != "msdos":
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Goal.PartitionType cannot be Extended for this"
                    " Extent.")
            part_type = parted.PARTITION_EXTENDED
        elif int(goal['PartitionType']) == part_types.Primary:
            part_type = parted.PARTITION_NORMAL
        elif int(goal['PartitionType']) == part_types.Logical:
            if device.format.labelType != "msdos":
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                         "Goal.PartitionType cannot be Logical for this"
                    " Extent.")
            part_type = parted.PARTITION_LOGICAL
        return part_type

    @cmpi_logging.trace_method
    def _lmi_create_partition(self, job, devicename, goal, size):
        """
            Create partition on given device with  given goal and size.
            Size can be null, which means the largest possible size.
            Return (retval, partition, size).
            This method is called from JobManager worker thread!
        """
        device = self.provider_manager.get_device_for_name(devicename)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "The devices disappeared: " + devicename)

        bootable = None
        part_type = None
        hidden = None
        primary = False

        if not isinstance(device.format,
                blivet.formats.disklabel.DiskLabel):
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "Cannot find partition table on the extent.")

        # check goal and set appropriate partition parameters
        if goal:
            bootable = goal['Bootable']
            hidden = goal['Hidden']
            part_type = self._calculate_partition_type(device, goal)
            if part_type is not None:
                if part_type == parted.PARTITION_LOGICAL:
                    primary = False
                else:
                    primary = True

        # check size and grow it if necessary
        if size is None:
            grow = True
            size = 1
        else:
            # check maximum size
            max_partition = self._get_max_partition_size(device, part_type)
            max_partition = max_partition * device.partedDevice.sectorSize
            if max_partition < size:
                ret = self.Values.LMI_CreateOrModifyPartition.Size_Not_Supported
                outparams = {'Size' : pywbem.Uint64(max_partition)}
                job.finish_method(
                        Job.STATE_FINISHED_OK,
                        return_value=ret,
                        return_type=Job.ReturnValueType.Uint32,
                        output_arguments=outparams,
                        affected_elements=[],
                        error=None)

            # Ok, the partition will fit. Continue.
            grow = False
            size = float(size) / units.MEGABYTE

        args = {
                'parents': [device],
                'size': size,
                'partType': part_type,
                'bootable': bootable,
                'grow': grow,
                'primary': primary
        }
        storage.log_storage_call("CREATE PARTITION", args)

        partition = self.storage.newPartition(**args)
        partition.disk = device

        if hidden is not None:
            if partition.flagAvailable(parted.PARTITION_HIDDEN):
                if hidden:
                    partition.setFlag(parted.PARTITION_HIDDEN)
            else:
                raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                        "Goal.Hidden cannot be set for this Extent.")

        # finally, do the dirty job
        action = blivet.deviceaction.ActionCreateDevice(partition)
        storage.do_storage_action(self.storage, [action])
        size = partition.size * units.MEGABYTE

        ret = self.Values.LMI_CreateOrModifyPartition\
                .Job_Completed_with_No_Error
        # re-read the partition from blivet, it should have all device links
        partition = self.storage.devicetree.getDeviceByPath(partition.path)
        partition_name = self.provider_manager.get_name_for_device(partition)
        outparams = {
                'Size': pywbem.Uint64(size),
                'Partition': partition_name
        }
        job.finish_method(
                Job.STATE_FINISHED_OK,
                return_value=ret,
                return_type=Job.ReturnValueType.Uint32,
                output_arguments=outparams,
                affected_elements=[partition_name, ],
                error=None)


    @cmpi_logging.trace_method
    def cim_method_lmi_createormodifypartition(self, env, object_name,
                                               param_partition=None,
                                               param_goal=None,
                                               param_extent=None,
                                               param_size=None):
        """
            Implements LMI_DiskPartitionConfigurationService.LMI_CreateOrModifyPartition()

            Create new partition on given extent.Partition modification is not
            yet supported.The implementation will select the best space to fit
            the partition, with all alignment rules etc. \nIf no Size
            parameter is provided, the largest possible partition is
            created.\nIf no Goal is provided and GPT partition is requested,
            normal partition is created. If no Goal is provided and MS-DOS
            partition is requested and there is extended partition already on
            the device, a logical partition is created. If there is no
            extended partition on the device and there are at most two primary
            partitions on the device, primary partition is created. If there
            is no extended partition and three primary partitions already
            exist, new extended partition with all remaining space is created
            and a logical partition with requested size is created.

            Keyword arguments:
            env -- Provider Environment (pycimmb.ProviderEnvironment)
            object_name -- A pywbem.CIMInstanceName or pywbem.CIMCLassName
                specifying the object on which the method LMI_CreateOrModifyPartition()
                should be invoked.
            param_partition --  The input parameter Partition (type REF (pywbem.CIMInstanceName(classname='CIM_GenericDiskPartition', ...))
                A reference an existing partition instance to modify or null to
                request a new partition.

            param_goal --  The input parameter Goal (type REF (pywbem.CIMInstanceName(classname='LMI_DiskPartitionConfigurationSetting', ...))
                Setting to be applied to created/modified partition.

            param_extent --  The input parameter extent (type REF (pywbem.CIMInstanceName(classname='CIM_StorageExtent', ...))
                A reference to the underlying extent the partition is base on.

            param_size --  The input parameter Size (type pywbem.Uint64)
                Requested size of the partition to create. If null when
                creating a partition, the larges possible partition is
                created.On output, the achieved size is returned.
        """
        self.check_instance(object_name)

        # remember input parameters for Job
        input_arguments = {
                'Partition' : pywbem.CIMProperty(name='Partition',
                        type='reference',
                        value=param_partition),
                'Goal' : pywbem.CIMProperty(name='Goal',
                        type='reference',
                        value=param_goal),
                'Extent': pywbem.CIMProperty(name='Extent',
                        type='reference',
                        value=param_extent),
                'Size': pywbem.CIMProperty(name='Size',
                        type='uint64',
                        value=param_size),
        }

        # check parameters
        if not param_partition and not param_extent:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Either Partition or extent parameter must be present.")

        goal = self._parse_goal(param_goal)
        (device, _unused) = self._parse_extent(param_extent, goal)
        partition = self._parse_partition(param_partition, device)

        if partition:
            # modify
            job_name = "MODIFY PARTITION %s" % (partition.path,)
            affected_elements = [param_partition]
        else:
            # create
            job_name = "CREATE PARTITION ON %s" % (device.path,)
            affected_elements = [param_extent]

        # prepare job
        job = Job(
                job_manager=self.job_manager,
                job_name=job_name,
                input_arguments=input_arguments,
                method_name='LMI_CreateOrModifyPartition',
                affected_elements=affected_elements,
                owning_element=self._get_instance_name())
        if partition:
            # modify
            job.set_execute_action(self._lmi_modify_partition,
                    job, param_partition, goal, param_size)

        else:
            # create
            job.set_execute_action(self._lmi_create_partition,
                    job, param_extent, goal, param_size)

        outparams = [ pywbem.CIMParameter(
                name='job',
                type='reference',
                value=job.get_name())]
        ret = self.Values.LMI_CreateOrModifyPartition\
                .Method_Parameters_Checked___Job_Started

        # enqueue the job
        self.job_manager.add_job(job)
        return (ret, outparams)

    def _delete_partition(self, job, devicename):
        """
            Delete given partition.
            This method is called from JobManager worker thread!
        """
        device = self.provider_manager.get_device_for_name(devicename)
        if not device:
            raise pywbem.CIMError(pywbem.CIM_ERR_FAILED,
                    "The devices disappeared: " + devicename)
        # Check the device is unused
        storage.assert_unused(self.storage, [device])

        action = blivet.ActionDestroyDevice(device)
        storage.do_storage_action(self.storage, [action])

        ret = self.Values.LMI_DeletePartition.Job_Completed_with_No_Error

        job.finish_method(
                Job.STATE_FINISHED_OK,
                return_value=ret,
                return_type=Job.ReturnValueType.Uint32,
                output_arguments=[],
                affected_elements=[],
                error=None)

    def cim_method_lmi_deletepartition(self, env, object_name,
                                       param_partition=None):
        """
        Implements LMI_DiskPartitionConfigurationService.LMI_DeletePartition()
        Delete partition.
        """
        self.check_instance(object_name)

        # remember input parameters for Job
        input_arguments = {
                'Partition' : pywbem.CIMProperty(name='Partition',
                        type='reference',
                        value=param_partition),
        }

        # check parameters
        if not param_partition:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Parameter Partition must be specified.")
        device = self._parse_partition(param_partition)
        # Check the device is unused
        storage.assert_unused(self.storage, [device])

        # prepare job
        job = Job(
                job_manager=self.job_manager,
                job_name="DELETE PARTITION %s" % (device.path,),
                input_arguments=input_arguments,
                method_name='LMI_DeletePartition',
                affected_elements=[param_partition],
                owning_element=self._get_instance_name())
        job.set_execute_action(self._delete_partition,
                job, param_partition)

        outparams = [ pywbem.CIMParameter(
                name='job',
                type='reference',
                value=job.get_name())]
        ret = self.Values.LMI_DeletePartition\
                .Method_Parameters_Checked___Job_Started

        # enqueue the job
        self.job_manager.add_job(job)
        return (ret, outparams)



    class Values(ServiceProvider.Values):
        class PartitioningSchemes(object):
            No_partitions_allowed = pywbem.Uint16(2)
            Volumes_may_be_partitioned_or_treated_as_whole = pywbem.Uint16(3)
            Volumes_must_be_partitioned = pywbem.Uint16(4)

        class SetPartitionStyle(object):
            Success = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            # DMTF_Reserved = ..
            # Extent_already_has_partition_table = 0x1000
            # Requested_Extent_too_large = 0x1001
            # Style_not_supported_by_Service = 0x1002
            # Method_Reserved = ..
            # Vendor_Specific = 0x8000..

        class CreateOrModifyPartition(object):
            Success = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            # DMTF_Reserved = ..
            # Overlap_Not_Supported = 0x1000
            # No_Available_Partitions = 0x1001
            # Specified_partition_not_on_specified_extent = 0x1002
            # Device_File_Name_not_valid = 0x1003
            # LogicalDisk_with_different_DeviceFileName_exists = 0x1004
            # Method_Reserved = ..
            # Vendor_Specific = 0x8000..

        class LMI_CreateOrModifyPartition(object):
            Job_Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Size_Not_Supported = pywbem.Uint32(4097)
            # Method_Reserved = 4098..32767
            # Vendor_Specific = 32768..65535

        class LMI_DeletePartition(object):
            Job_Completed_with_No_Error = pywbem.Uint32(0)
            Not_Supported = pywbem.Uint32(1)
            Unknown = pywbem.Uint32(2)
            Timeout = pywbem.Uint32(3)
            Failed = pywbem.Uint32(4)
            Invalid_Parameter = pywbem.Uint32(5)
            In_Use = pywbem.Uint32(6)
            # DMTF_Reserved = ..
            Method_Parameters_Checked___Job_Started = pywbem.Uint32(4096)
            Size_Not_Supported = pywbem.Uint32(4097)
            # Method_Reserved = 4098..32767
            # Vendor_Specific = 32768..65535



# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for FormatProvider.

LMI_PVFormatProvider
--------------------

.. autoclass:: LMI_PVFormatProvider
    :members:

"""

from lmi.storage.FormatProvider import FormatProvider
import blivet.formats.lvmpv
import lmi.providers.cmpi_logging as cmpi_logging

class LMI_PVFormatProvider(FormatProvider):
    """
        Provider of MD RAID format on a device.
    """
    @cmpi_logging.trace_method
    def __init__(self, *args, **kwargs):
        super(LMI_PVFormatProvider, self).__init__(
                "LMI_PVFormat",
                "lvmpv",
                *args, **kwargs)

    @cmpi_logging.trace_method
    def provides_format(self, device, fmt):
        if  isinstance(fmt, blivet.formats.lvmpv.LVMPhysicalVolume):
            return True
        return False

    @cmpi_logging.trace_method
    def get_instance(self, env, model, fmt=None):
        """
            Get instance.
        """
        model = super(LMI_PVFormatProvider, self).get_instance(
                env, model, fmt)
        if not fmt:
            (device, fmt) = self.get_format_for_id(model['Name'])
        if fmt.uuid:
            model['UUID'] = fmt.uuid
        return model



# Copyright (C) 2012 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# -*- coding: utf-8 -*-
"""
Module for ProviderManager class.

ProviderManager
---------------

.. autoclass:: ProviderManager
    :members:

"""

import lmi.providers.cmpi_logging as cmpi_logging

class ProviderManager(object):
    """
        Simple class containing references to various providers to simplify
        various tasks.

        The manager holds reference to all device providers of Anaconda
        StorageDevice subclasses and all LMI_*Setting providers.
        The manager can then find a CIM provider for a
        StorageDevice instance and find provider or StorageDevice instance
        for CIM InstanceName.
        The major benefit of this manager is to convert Anaconda StorageDevice
        instance to CIM InstanceName and back. Therefore various associations
        can easily get their Antecendent/Dependent InstanceNames.

        The device providers must be registered by add_device_provider().
        The device providers must be subclasses of DeviceProvider class.

        The manager holds references to LMI_*Setting providers. For example
        LMI_*Service providers need access to settings, so they can use this
        manager to get them.
        The setting providers must be registered by add_setting_provider().
        The setting providers must be subclasses of SettingProvider class.

        The manager holds references to LMI_*Service providers, so
        the LMI_HostedService can easily enumerate all services.
        The service providers must be registered by add_service_provider().
        The service providers must be subclasses of ServiceProvider class.
    """

    @cmpi_logging.trace_method
    def __init__(self):
        self.device_providers = []
        self.setting_providers = []
        self.service_providers = []
        self.capabilities_providers = []
        self.format_providers = []

    @cmpi_logging.trace_method
    def add_device_provider(self, provider):
        """
            Add new device provider to the manager.
        """
        self.device_providers.append(provider)

    @cmpi_logging.trace_method
    def add_setting_provider(self, provider):
        """
            Add new setting provider to the manager.
        """
        self.setting_providers.append(provider)

    @cmpi_logging.trace_method
    def add_service_provider(self, provider):
        """
            Add new service provider to the manager.
        """
        self.service_providers.append(provider)

    @cmpi_logging.trace_method
    def add_capabilities_provider(self, provider):
        """
            Add new service provider to the manager.
        """
        self.capabilities_providers.append(provider)

    @cmpi_logging.trace_method
    def add_format_provider(self, provider):
        """
            Add new service provider to the manager.
        """
        self.format_providers.append(provider)

    @cmpi_logging.trace_method
    def get_device_provider_for_name(self, object_name):
        """
            Return provider for given CIM InstanceName.
            Return None if no such provider is registered.
        """
        for provider in self.device_providers:
            if provider.provides_name(object_name):
                return provider
        return None

    @cmpi_logging.trace_method
    def get_device_for_name(self, object_name):
        """
            Return Anaconda StorageDevice for given CIM InstanceName.
            Return None if no device exist.
        """
        provider = self.get_device_provider_for_name(object_name)
        if provider:
            return provider.get_device_for_name(object_name)
        return None

    @cmpi_logging.trace_method
    def get_provider_for_device(self, device):
        """
            Return provider for given Anaconda StorageDevice.
            Return None if no such provider is registered.
        """
        for provider in self.device_providers:
            if provider.provides_device(device):
                return provider
        return None

    @cmpi_logging.trace_method
    def get_name_for_device(self, device):
        """
            Return CIM InstanceName for given Anaconda StorageDevice.
            Return None if no device exist.
        """
        provider = self.get_provider_for_device(device)
        if provider:
            return provider.get_name_for_device(device)
        return None

    @cmpi_logging.trace_method
    def get_provider_for_format(self, device, fmt):
        """
            Return FormatProvider for given DeviceFormat subclass
        """
        for prov in self.format_providers:
            if prov.provides_format(device, fmt):
                return prov
        return None

    @cmpi_logging.trace_method
    def get_provider_for_format_name(self, object_name):
        """
            Return provider for given CIM InstanceName of LMI_FileSystemFormat
            subclass.
            Return None if no such provider is registered.
        """
        for provider in self.format_providers:
            if provider.provides_name(object_name):
                return provider
        return None

    @cmpi_logging.trace_method
    def get_name_for_format(self, device, fmt):
        """
            Return CIM InstanceName for given Blivet Format.
            Return None if no format exist.
        """
        provider = self.get_provider_for_format(device, fmt)
        if provider:
            return provider.get_name_for_format(device, fmt)
        return None

    @cmpi_logging.trace_method
    def get_setting_for_id(self, instance_id, setting_classname=None):
        """
            Return Setting instance for given InstanceID.
            If setting_classname is not None, it also checks that the
            setting classname equals setting_classname.

            Return None if there is no such instance.
        """
        # parse the instance id
        parts = instance_id.split(":")
        if len(parts) != 3:
            return None
        if parts[0] != "LMI":
            return None
        classname = parts[1]
        if setting_classname and setting_classname != classname:
            return None
        for provider in self.setting_providers:
            if provider.setting_classname == classname:
                return provider.find_instance(instance_id)

    @cmpi_logging.trace_method
    def get_service_providers(self):
        """ Return list of registered service providers."""
        return self.service_providers

    @cmpi_logging.trace_method
    def get_capabilities_provider_for_class(self, classname):
        """ Return list of registered capabilities providers."""
        for provider in self.capabilities_providers:
            if provider.classname == classname:
                return provider
        return None
